@@ -17,7 +17,6 @@ limitations under the License.
 package v1
 
 import (
-	"errors"
 	"fmt"
 	"os"
 	"strings"
@@ -31,16 +30,23 @@ import (
 	quaycontext "github.com/quay/quay-operator/pkg/context"
 )
 
+// QuayVersion represents a quay version as a string. Normally this is set using semantic
+// versioning, e.g. 3.7.0.
 type QuayVersion string
 
+// QuayVersionCurrent holds the current quay version as captured by means of an environment
+// variable. This can be used to identify a version upgrade.
 var QuayVersionCurrent QuayVersion = QuayVersion(os.Getenv("QUAY_VERSION"))
 
+// ComponentKind holds a component type, e.g. "clair", "postgres", etc.
 type ComponentKind string
 
+// Follow a list of constants representing all supported components.
 const (
-	ComponentBase          ComponentKind = "base"
+	ComponentQuay          ComponentKind = "quay"
 	ComponentPostgres      ComponentKind = "postgres"
 	ComponentClair         ComponentKind = "clair"
+	ComponentClairPostgres ComponentKind = "clairpostgres"
 	ComponentRedis         ComponentKind = "redis"
 	ComponentHPA           ComponentKind = "horizontalpodautoscaler"
 	ComponentObjectStorage ComponentKind = "objectstorage"
@@ -50,7 +56,9 @@ const (
 	ComponentTLS           ComponentKind = "tls"
 )
 
-var allComponents = []ComponentKind{
+// AllComponents holds a list of all supported components.
+var AllComponents = []ComponentKind{
+	ComponentQuay,
 	ComponentPostgres,
 	ComponentClair,
 	ComponentRedis,
@@ -60,6 +68,7 @@ var allComponents = []ComponentKind{
 	ComponentMirror,
 	ComponentMonitoring,
 	ComponentTLS,
+	ComponentClairPostgres,
 }
 
 var requiredComponents = []ComponentKind{
@@ -75,6 +84,20 @@ var supportsVolumeOverride = []ComponentKind{
 	ComponentClair,
 }
 
+var supportsEnvOverride = []ComponentKind{
+	ComponentQuay,
+	ComponentClair,
+	ComponentMirror,
+	ComponentPostgres,
+	ComponentRedis,
+}
+
+var supportsReplicasOverride = []ComponentKind{
+	ComponentClair,
+	ComponentMirror,
+	ComponentQuay,
+}
+
 const (
 	ManagedKeysName         = "quay-registry-managed-secret-keys"
 	QuayConfigTLSSecretName = "quay-config-tls"
@@ -83,7 +106,8 @@ const (
 
 // QuayRegistrySpec defines the desired state of QuayRegistry.
 type QuayRegistrySpec struct {
-	// ConfigBundleSecret is the name of the Kubernetes `Secret` in the same namespace which contains the base Quay config and extra certs.
+	// ConfigBundleSecret is the name of the Kubernetes `Secret` in the same namespace
+	// which contains the base Quay config and extra certs.
 	ConfigBundleSecret string `json:"configBundleSecret,omitempty"`
 	// Components declare how the Operator should handle backing Quay services.
 	Components []Component `json:"components,omitempty"`
@@ -103,6 +127,8 @@ type Component struct {
 // Override describes configuration overrides for the given managed component
 type Override struct {
 	VolumeSize *resource.Quantity `json:"volumeSize,omitempty"`
+	Env        []corev1.EnvVar    `json:"env,omitempty" patchStrategy:"merge" patchMergeKey:"name"`
+	Replicas   *int32             `json:"replicas,omitempty"`
 }
 
 type ConditionType string
@@ -113,9 +139,10 @@ const (
 	ConditionTypeAvailable      ConditionType = "Available"
 	ConditionTypeRolloutBlocked ConditionType = "RolloutBlocked"
 	ConditionComponentsCreated  ConditionType = "ComponentsCreated"
-	ComponentBaseReady          ConditionType = "ComponentBaseReady"
+	ComponentQuayReady          ConditionType = "ComponentQuayReady"
 	ComponentPostgresReady      ConditionType = "ComponentPostgresReady"
 	ComponentClairReady         ConditionType = "ComponentClairReady"
+	ComponentClairPostgresReady ConditionType = "ComponentClairPostgresReady"
 	ComponentRedisReady         ConditionType = "ComponentRedisReady"
 	ComponentHPAReady           ConditionType = "ComponentHPAReady"
 	ComponentObjectStorageReady ConditionType = "ComponentObjectStorageReady"
@@ -242,6 +269,35 @@ type QuayRegistryList struct {
 	Items           []QuayRegistry `json:"items"`
 }
 
+// MigrationsRunning returns true if the status for provided QuayRegistry indicates that
+// the database migrations are running.
+func MigrationsRunning(quay *QuayRegistry) bool {
+	created := GetCondition(quay.Status.Conditions, ConditionComponentsCreated)
+	if created == nil {
+		return false
+	}
+	return created.Reason == ConditionReasonMigrationsInProgress
+}
+
+// FlaggedForDeletion returns a boolean indicating if provided QuayRegistry object has
+// been flagged for deletion.
+func FlaggedForDeletion(quay *QuayRegistry) bool {
+	return quay.GetDeletionTimestamp() != nil
+}
+
+// NeedsBundleSecret returns if provided QuayRegistry has not a config bundle secret
+// populated on its Spec.ConfigBundleSecret property.
+func NeedsBundleSecret(quay *QuayRegistry) bool {
+	return quay.Spec.ConfigBundleSecret == ""
+}
+
+// ComponentSupportsConfigWhenManaged returns true if provided component can live with
+// being Managed AND containing a custom user config provided through the config bundle
+// secret.
+func ComponentSupportsConfigWhenManaged(cmp Component) bool {
+	return cmp.Kind == ComponentRoute || cmp.Kind == ComponentMirror
+}
+
 func EnsureComponents(components []Component) []Component {
 	return append(components, components[0])[1 : len(components)+1]
 }
@@ -269,9 +325,9 @@ func ComponentsMatch(firstComponents, secondComponents []Component) bool {
 
 // ComponentIsManaged returns whether the given component is managed or not.
 func ComponentIsManaged(components []Component, name ComponentKind) bool {
-	// we do not expose Base component through the CRD, as it represents the base deployment
-	// of a quay registry and will always be managed.
-	if name == ComponentBase {
+	// we do not expose Quay component through the CRD, as it represents the Quay deployment
+	// and will always be managed.
+	if name == ComponentQuay {
 		return true
 	}
 
@@ -294,58 +350,94 @@ func RequiredComponent(component ComponentKind) bool {
 }
 
 // EnsureDefaultComponents adds any `Components` which are missing from `Spec.Components`.
-// Returns an error if a component was declared as managed but is not supported in the current k8s cluster.
-func EnsureDefaultComponents(ctx *quaycontext.QuayRegistryContext, quay *QuayRegistry) (*QuayRegistry, error) {
-	updatedQuay := quay.DeepCopy()
-	if updatedQuay.Spec.Components == nil {
-		updatedQuay.Spec.Components = []Component{}
+// Returns an error if a component was declared as managed but is not supported in the current
+// k8s cluster.
+func EnsureDefaultComponents(ctx *quaycontext.QuayRegistryContext, quay *QuayRegistry) error {
+	if quay.Spec.Components == nil {
+		quay.Spec.Components = []Component{}
 	}
 
-	type componentCheck struct {
+	type check struct {
 		check func() bool
 		msg   string
 	}
-	componentChecks := map[ComponentKind]componentCheck{
-		ComponentRoute:         {func() bool { return ctx.SupportsRoutes }, "cannot use `route` component when `Route` API not available"},
-		ComponentTLS:           {func() bool { return ctx.SupportsRoutes }, "cannot use `tls` component when `Route` API not available"},
-		ComponentObjectStorage: {func() bool { return ctx.SupportsObjectStorage }, "cannot use `ObjectStorage` component when `ObjectStorage` API not available"},
-		ComponentMonitoring:    {func() bool { return ctx.SupportsMonitoring }, "cannot use `monitoring` component when `Prometheus` API not available"},
+	checks := map[ComponentKind]check{
+		ComponentRoute: {
+			check: func() bool { return ctx.SupportsRoutes },
+			msg:   "Route API not available",
+		},
+		ComponentTLS: {
+			check: func() bool { return ctx.SupportsRoutes },
+			msg:   "Route API not available",
+		},
+		ComponentObjectStorage: {
+			check: func() bool { return ctx.SupportsObjectStorage },
+			msg:   "ObjectStorage API not available",
+		},
+		ComponentMonitoring: {
+			check: func() bool { return ctx.SupportsMonitoring },
+			msg:   "Prometheus API not available",
+		},
 	}
 
-	componentManaged := map[ComponentKind]componentCheck{
+	componentManaged := map[ComponentKind]check{
 		ComponentTLS: {
 			check: func() bool { return ctx.TLSCert == nil && ctx.TLSKey == nil },
 		},
 	}
 
-	for _, component := range allComponents {
-		componentCheck, checkExists := componentChecks[component]
-		if (checkExists && !componentCheck.check()) && ComponentIsManaged(quay.Spec.Components, component) {
-			return quay, errors.New(componentCheck.msg)
-		}
-
-		found := false
-		for _, declaredComponent := range quay.Spec.Components {
-			if component == declaredComponent.Kind {
-				found = true
-				break
+	for _, cmp := range AllComponents {
+		ccheck, checkexists := checks[cmp]
+		if checkexists {
+			// if there is a check registered for the component we run it, if the
+			// check fails and the component is managed then we have a problem with
+			// the current components configuration. returns the check error.
+			if !ccheck.check() && ComponentIsManaged(quay.Spec.Components, cmp) {
+				return fmt.Errorf(
+					"Error validatingcomponent %s: %s", cmp, ccheck.msg,
+				)
 			}
 		}
 
-		managed := !checkExists || componentCheck.check()
-		if _, ok := componentManaged[component]; ok {
-			managed = managed && componentManaged[component].check()
+		// if the component has already been declared in the QuayRegistry object we can
+		// just return as there is nothing we need to do.
+		var found bool
+		for i, declaredComponent := range quay.Spec.Components {
+			if cmp != declaredComponent.Kind {
+				continue
+			}
+
+			// we disregard whatever the user has defined for Quay component, this
+			// is a component that can't be unmanaged so if user sets it to unmanaged
+			// we are going to roll it back to managed.
+			if declaredComponent.Kind == ComponentQuay {
+				quay.Spec.Components[i].Managed = true
+			}
+
+			found = true
+			break
+		}
+		if found {
+			continue
 		}
 
-		if !found {
-			updatedQuay.Spec.Components = append(updatedQuay.Spec.Components, Component{
-				Kind:    component,
-				Managed: managed,
-			})
+		// the component management status is set to true if the check for the component
+		// has passed.
+		managed := !checkexists || ccheck.check()
+		if _, ok := componentManaged[cmp]; ok {
+			managed = managed && componentManaged[cmp].check()
 		}
+
+		quay.Spec.Components = append(
+			quay.Spec.Components,
+			Component{
+				Kind:    cmp,
+				Managed: managed,
+			},
+		)
 	}
 
-	return updatedQuay, nil
+	return nil
 }
 
 // ValidateOverrides validates that the overrides set for each component are valid.
@@ -357,42 +449,71 @@ func ValidateOverrides(quay *QuayRegistry) error {
 			continue
 		}
 
-		// If the component is unmanaged, we cannot set overrides
-		if !ComponentIsManaged(quay.Spec.Components, component.Kind) {
-			if component.Overrides.VolumeSize != nil {
-				return errors.New("cannot set overrides on unmanaged component " + string(component.Kind))
+		hasvolume := component.Overrides.VolumeSize != nil
+		hasreplicas := component.Overrides.Replicas != nil
+		hasenvvar := len(component.Overrides.Env) > 0
+		hasoverride := hasvolume || hasenvvar || hasreplicas
+
+		if hasoverride && !ComponentIsManaged(quay.Spec.Components, component.Kind) {
+			return fmt.Errorf("cannot set overrides on unmanaged %s", component.Kind)
+		}
+
+		if hasreplicas && ComponentIsManaged(quay.Spec.Components, ComponentHPA) {
+			// with managed HPA we only accept zero as an override for the number
+			// of replicas. we can't compete with HPA except when scaling down.
+			if *component.Overrides.Replicas != 0 {
+				return fmt.Errorf("cannot override replicas with managed HPA")
 			}
 		}
 
 		// Check that component supports override
-		if component.Overrides.VolumeSize != nil && !ComponentSupportsOverride(component.Kind, "volumeSize") {
-			return fmt.Errorf("component %s does not support volumeSize overrides", component.Kind)
+		if hasvolume && !ComponentSupportsOverride(component.Kind, "volumeSize") {
+			return fmt.Errorf(
+				"component %s does not support volumeSize overrides",
+				component.Kind,
+			)
 		}
 
+		if hasenvvar && !ComponentSupportsOverride(component.Kind, "env") {
+			return fmt.Errorf(
+				"component %s does not support env overrides",
+				component.Kind,
+			)
+		}
+
+		if hasreplicas && !ComponentSupportsOverride(component.Kind, "replicas") {
+			return fmt.Errorf(
+				"component %s does not support replicas overrides",
+				component.Kind,
+			)
+		}
 	}
 
 	return nil
-
 }
 
-// EnsureRegistryEndpoint sets the `status.registryEndpoint` field and returns `ok` if it was unchanged.
-func EnsureRegistryEndpoint(ctx *quaycontext.QuayRegistryContext, quay *QuayRegistry, config map[string]interface{}) (*QuayRegistry, bool) {
-	updatedQuay := quay.DeepCopy()
-
+// EnsureRegistryEndpoint sets the `status.registryEndpoint` field and returns `ok` if it was
+// unchanged.
+func EnsureRegistryEndpoint(
+	qctx *quaycontext.QuayRegistryContext, quay *QuayRegistry, config map[string]interface{},
+) bool {
+	orig := quay.Status.RegistryEndpoint
 	if config == nil {
 		config = map[string]interface{}{}
 	}
 
 	if serverHostname, ok := config["SERVER_HOSTNAME"]; ok {
-		updatedQuay.Status.RegistryEndpoint = "https://" + serverHostname.(string)
-	} else if ctx.SupportsRoutes {
-		updatedQuay.Status.RegistryEndpoint = "https://" + strings.Join([]string{
-			strings.Join([]string{quay.GetName(), "quay", quay.GetNamespace()}, "-"),
-			ctx.ClusterHostname},
-			".")
+		quay.Status.RegistryEndpoint = "https://" + serverHostname.(string)
+	} else if qctx.SupportsRoutes {
+		quay.Status.RegistryEndpoint = fmt.Sprintf(
+			"https://%s-quay-%s.%s",
+			quay.GetName(),
+			quay.GetNamespace(),
+			qctx.ClusterHostname,
+		)
 	}
 
-	return updatedQuay, quay.Status.RegistryEndpoint == updatedQuay.Status.RegistryEndpoint
+	return quay.Status.RegistryEndpoint == orig
 }
 
 // EnsureConfigEditorEndpoint sets the `status.configEditorEndpoint` field. If routes are
@@ -437,33 +558,37 @@ func Owns(quay QuayRegistry, obj client.Object) bool {
 	return false
 }
 
-// EnsureOwnerReference adds an `ownerReference` to the given object if it does not already have one.
-func EnsureOwnerReference(quay *QuayRegistry, obj client.Object) (client.Object, error) {
-	objectMeta, err := meta.Accessor(obj)
-	if err != nil {
-		return nil, err
-	}
-
-	hasOwnerRef := false
-	for _, ownerRef := range objectMeta.GetOwnerReferences() {
-		if ownerRef.Name == quay.GetName() &&
-			ownerRef.Kind == "QuayRegistry" &&
-			ownerRef.APIVersion == GroupVersion.String() &&
-			ownerRef.UID == quay.UID {
-			hasOwnerRef = true
+// EnsureOwnerReference adds an `ownerReference` to the given object if it does not already
+// have one.
+func EnsureOwnerReference(quay *QuayRegistry, obj client.Object) client.Object {
+	for _, ownerRef := range obj.GetOwnerReferences() {
+		if ownerRef.Kind != "QuayRegistry" {
+			continue
 		}
+		if ownerRef.APIVersion != GroupVersion.String() {
+			continue
+		}
+		if ownerRef.Name != quay.GetName() {
+			continue
+		}
+		if ownerRef.UID != quay.UID {
+			continue
+		}
+		return obj
 	}
 
-	if !hasOwnerRef {
-		objectMeta.SetOwnerReferences(append(objectMeta.GetOwnerReferences(), metav1.OwnerReference{
-			APIVersion: GroupVersion.String(),
-			Kind:       "QuayRegistry",
-			Name:       quay.GetName(),
-			UID:        quay.GetUID(),
-		}))
-	}
-
-	return obj, nil
+	obj.SetOwnerReferences(
+		append(
+			obj.GetOwnerReferences(),
+			metav1.OwnerReference{
+				APIVersion: GroupVersion.String(),
+				Kind:       "QuayRegistry",
+				Name:       quay.GetName(),
+				UID:        quay.GetUID(),
+			},
+		),
+	)
+	return obj
 }
 
 // RemoveOwnerReference removes the `ownerReference` of `QuayRegistry` on the given object.
@@ -505,6 +630,8 @@ func FieldGroupNameFor(cmp ComponentKind) (string, error) {
 		return "SecurityScanner", nil
 	case ComponentPostgres:
 		return "Database", nil
+	case ComponentClairPostgres:
+		return "", nil
 	case ComponentRedis:
 		return "Redis", nil
 	case ComponentObjectStorage:
@@ -519,6 +646,8 @@ func FieldGroupNameFor(cmp ComponentKind) (string, error) {
 		return "", nil
 	case ComponentTLS:
 		return "", nil
+	case ComponentQuay:
+		return "", nil
 	default:
 		return "", fmt.Errorf("unknown component: %q", cmp)
 	}
@@ -548,22 +677,111 @@ func FieldGroupNamesForManagedComponents(quay *QuayRegistry) ([]string, error) {
 
 // ComponentSupportsOverride returns whether or not a given component supports the given override.
 func ComponentSupportsOverride(component ComponentKind, override string) bool {
+	var components []ComponentKind
 
-	// Using a switch statement for possible implementation of future overrides
 	switch override {
 	case "volumeSize":
-		for _, c := range supportsVolumeOverride {
-			if c == component {
-				return true
-			}
-		}
-	default:
-		return false
+		components = supportsVolumeOverride
+	case "env":
+		components = supportsEnvOverride
+	case "replicas":
+		components = supportsReplicasOverride
 	}
 
+	for _, cmp := range components {
+		if component == cmp {
+			return true
+		}
+	}
 	return false
 }
 
 func init() {
 	SchemeBuilder.Register(&QuayRegistry{}, &QuayRegistryList{})
 }
+
+// GetReplicasOverrideForComponent returns the overrides set by the user for the provided
+// component. Returns nil if not set.
+func GetReplicasOverrideForComponent(quay *QuayRegistry, kind ComponentKind) *int32 {
+	for _, cmp := range quay.Spec.Components {
+		if cmp.Kind != kind {
+			continue
+		}
+
+		if cmp.Overrides == nil {
+			return nil
+		}
+
+		return cmp.Overrides.Replicas
+	}
+	return nil
+}
+
+// GetVolumeSizeOverrideForComponent returns the volume size overrides set by the user for the
+// provided component. Returns nil if not set.
+func GetVolumeSizeOverrideForComponent(
+	quay *QuayRegistry, kind ComponentKind,
+) (qt *resource.Quantity) {
+	for _, component := range quay.Spec.Components {
+		if component.Kind != kind {
+			continue
+		}
+
+		if component.Overrides != nil && component.Overrides.VolumeSize != nil {
+			qt = component.Overrides.VolumeSize
+		}
+		return
+	}
+	return
+}
+
+// GetEnvOverrideForComponent return the environment variables overrides for the provided
+// component, nil is returned if not defined.
+func GetEnvOverrideForComponent(quay *QuayRegistry, kind ComponentKind) []corev1.EnvVar {
+	for _, cmp := range quay.Spec.Components {
+		if cmp.Kind != kind {
+			continue
+		}
+
+		if cmp.Overrides == nil {
+			return nil
+		}
+
+		return cmp.Overrides.Env
+	}
+	return nil
+}
+
+// RemoveUnusedConditions is used to trim off conditions created by previous releases of this
+// operator that are not used anymore.
+func RemoveUnusedConditions(quay *QuayRegistry) {
+	validConditionTypes := []ConditionType{
+		ConditionTypeAvailable,
+		ConditionTypeRolloutBlocked,
+		ConditionComponentsCreated,
+		ComponentQuayReady,
+		ComponentPostgresReady,
+		ComponentClairReady,
+		ComponentClairPostgresReady,
+		ComponentRedisReady,
+		ComponentHPAReady,
+		ComponentObjectStorageReady,
+		ComponentRouteReady,
+		ComponentMirrorReady,
+		ComponentMonitoringReady,
+		ComponentTLSReady,
+	}
+
+	newconds := []Condition{}
+	for _, qcond := range quay.Status.Conditions {
+		for _, valid := range validConditionTypes {
+			if qcond.Type != valid {
+				continue
+			}
+			newconds = append(newconds, qcond)
+			break
+		}
+	}
+
+	quay.Status.Conditions = newconds
+}
