@@ -8,14 +8,16 @@ import (
 
 	appsv1 "k8s.io/api/apps/v1"
 	asv2b2 "k8s.io/api/autoscaling/v2beta2"
+	batchv1 "k8s.io/api/batch/v1"
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
+	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 
-	monv1 "github.com/coreos/prometheus-operator/pkg/apis/monitoring/v1"
 	ocsv1a1 "github.com/kube-object-storage/lib-bucket-provisioner/pkg/apis/objectbucket.io/v1alpha1"
 	routev1 "github.com/openshift/api/route/v1"
+	monv1 "github.com/prometheus-operator/prometheus-operator/pkg/apis/monitoring/v1"
 
 	qv1 "github.com/quay/quay-operator/apis/quay/v1"
 )
@@ -23,7 +25,7 @@ import (
 func TestEvaluate(t *testing.T) {
 	for _, tt := range []struct {
 		name  string
-		objs  []runtime.Object
+		objs  []client.Object
 		quay  qv1.QuayRegistry
 		conds []qv1.Condition
 	}{
@@ -43,6 +45,10 @@ func TestEvaluate(t *testing.T) {
 							Kind:    qv1.ComponentClair,
 							Managed: true,
 						},
+						{
+							Kind:    qv1.ComponentClairPostgres,
+							Managed: true,
+						},
 						{
 							Kind:    qv1.ComponentRedis,
 							Managed: true,
@@ -109,6 +115,12 @@ func TestEvaluate(t *testing.T) {
 					Type:    qv1.ComponentClairReady,
 					Status:  metav1.ConditionFalse,
 					Reason:  qv1.ConditionReasonComponentNotReady,
+					Message: "Deployment registry-clair-app not found",
+				},
+				{
+					Type:    qv1.ComponentClairPostgresReady,
+					Status:  metav1.ConditionFalse,
+					Reason:  qv1.ConditionReasonComponentNotReady,
 					Message: "Deployment registry-clair-postgres not found",
 				},
 				{
@@ -124,23 +136,23 @@ func TestEvaluate(t *testing.T) {
 					Message: "Redis deployment not found",
 				},
 				{
-					Type:   qv1.ComponentBaseReady,
+					Type:   qv1.ComponentQuayReady,
 					Status: metav1.ConditionFalse,
 					Reason: qv1.ConditionReasonComponentNotReady,
 					Message: "Awaiting for component postgres," +
-						"objectstorage,clair,tls,redis to become " +
+						"objectstorage,clair,clairpostgres,tls,redis to become " +
 						"available",
 				},
 				{
 					Type:    qv1.ComponentMirrorReady,
 					Status:  metav1.ConditionFalse,
 					Reason:  qv1.ConditionReasonComponentNotReady,
-					Message: "Awaiting for component base to become available",
+					Message: "Awaiting for component quay to become available",
 				},
 			},
 		},
 		{
-			name: "base unhealthy",
+			name: "quay unhealthy",
 			quay: qv1.QuayRegistry{
 				ObjectMeta: metav1.ObjectMeta{
 					Name: "registry",
@@ -161,6 +173,10 @@ func TestEvaluate(t *testing.T) {
 							Kind:    qv1.ComponentClair,
 							Managed: true,
 						},
+						{
+							Kind:    qv1.ComponentClairPostgres,
+							Managed: true,
+						},
 						{
 							Kind:    qv1.ComponentRedis,
 							Managed: true,
@@ -188,7 +204,15 @@ func TestEvaluate(t *testing.T) {
 					},
 				},
 			},
-			objs: []runtime.Object{
+			objs: []client.Object{
+				&batchv1.Job{
+					ObjectMeta: metav1.ObjectMeta{
+						Name: "registry-quay-app-upgrade",
+					},
+					Status: batchv1.JobStatus{
+						Succeeded: 1,
+					},
+				},
 				&routev1.Route{
 					ObjectMeta: metav1.ObjectMeta{
 						OwnerReferences: []metav1.OwnerReference{
@@ -291,6 +315,7 @@ func TestEvaluate(t *testing.T) {
 						},
 					},
 					Status: appsv1.DeploymentStatus{
+						AvailableReplicas: 1,
 						Conditions: []appsv1.DeploymentCondition{
 							{
 								Type:    appsv1.DeploymentAvailable,
@@ -328,6 +353,7 @@ func TestEvaluate(t *testing.T) {
 						},
 					},
 					Status: appsv1.DeploymentStatus{
+						AvailableReplicas: 1,
 						Conditions: []appsv1.DeploymentCondition{
 							{
 								Type:    appsv1.DeploymentAvailable,
@@ -350,6 +376,7 @@ func TestEvaluate(t *testing.T) {
 						},
 					},
 					Status: appsv1.DeploymentStatus{
+						AvailableReplicas: 1,
 						Conditions: []appsv1.DeploymentCondition{
 							{
 								Type:    appsv1.DeploymentAvailable,
@@ -377,6 +404,7 @@ func TestEvaluate(t *testing.T) {
 						},
 					},
 					Status: appsv1.DeploymentStatus{
+						AvailableReplicas: 1,
 						Conditions: []appsv1.DeploymentCondition{
 							{
 								Type:    appsv1.DeploymentAvailable,
@@ -424,6 +452,12 @@ func TestEvaluate(t *testing.T) {
 					Reason:  qv1.ConditionReasonComponentReady,
 					Message: "Clair component healthy",
 				},
+				{
+					Type:    qv1.ComponentClairPostgresReady,
+					Status:  metav1.ConditionTrue,
+					Reason:  qv1.ConditionReasonComponentReady,
+					Message: "ClairPostgres component healthy",
+				},
 				{
 					Type:    qv1.ComponentTLSReady,
 					Status:  metav1.ConditionTrue,
@@ -437,7 +471,7 @@ func TestEvaluate(t *testing.T) {
 					Message: "Deployment registry-quay-redis healthy",
 				},
 				{
-					Type:    qv1.ComponentBaseReady,
+					Type:    qv1.ComponentQuayReady,
 					Status:  metav1.ConditionFalse,
 					Reason:  qv1.ConditionReasonComponentNotReady,
 					Message: "Deployment registry-quay-app not found",
@@ -446,12 +480,12 @@ func TestEvaluate(t *testing.T) {
 					Type:    qv1.ComponentMirrorReady,
 					Status:  metav1.ConditionFalse,
 					Reason:  qv1.ConditionReasonComponentNotReady,
-					Message: "Awaiting for component base to become available",
+					Message: "Awaiting for component quay to become available",
 				},
 			},
 		},
 		{
-			name: "base healthy",
+			name: "quay healthy",
 			quay: qv1.QuayRegistry{
 				ObjectMeta: metav1.ObjectMeta{
 					Name: "registry",
@@ -472,6 +506,10 @@ func TestEvaluate(t *testing.T) {
 							Kind:    qv1.ComponentClair,
 							Managed: true,
 						},
+						{
+							Kind:    qv1.ComponentClairPostgres,
+							Managed: true,
+						},
 						{
 							Kind:    qv1.ComponentRedis,
 							Managed: true,
@@ -499,7 +537,15 @@ func TestEvaluate(t *testing.T) {
 					},
 				},
 			},
-			objs: []runtime.Object{
+			objs: []client.Object{
+				&batchv1.Job{
+					ObjectMeta: metav1.ObjectMeta{
+						Name: "registry-quay-app-upgrade",
+					},
+					Status: batchv1.JobStatus{
+						Succeeded: 1,
+					},
+				},
 				&routev1.Route{
 					ObjectMeta: metav1.ObjectMeta{
 						OwnerReferences: []metav1.OwnerReference{
@@ -602,6 +648,7 @@ func TestEvaluate(t *testing.T) {
 						},
 					},
 					Status: appsv1.DeploymentStatus{
+						AvailableReplicas: 1,
 						Conditions: []appsv1.DeploymentCondition{
 							{
 								Type:    appsv1.DeploymentAvailable,
@@ -639,6 +686,7 @@ func TestEvaluate(t *testing.T) {
 						},
 					},
 					Status: appsv1.DeploymentStatus{
+						AvailableReplicas: 1,
 						Conditions: []appsv1.DeploymentCondition{
 							{
 								Type:    appsv1.DeploymentAvailable,
@@ -661,6 +709,7 @@ func TestEvaluate(t *testing.T) {
 						},
 					},
 					Status: appsv1.DeploymentStatus{
+						AvailableReplicas: 1,
 						Conditions: []appsv1.DeploymentCondition{
 							{
 								Type:    appsv1.DeploymentAvailable,
@@ -688,6 +737,7 @@ func TestEvaluate(t *testing.T) {
 						},
 					},
 					Status: appsv1.DeploymentStatus{
+						AvailableReplicas: 1,
 						Conditions: []appsv1.DeploymentCondition{
 							{
 								Type:    appsv1.DeploymentAvailable,
@@ -710,6 +760,7 @@ func TestEvaluate(t *testing.T) {
 						},
 					},
 					Status: appsv1.DeploymentStatus{
+						AvailableReplicas: 1,
 						Conditions: []appsv1.DeploymentCondition{
 							{
 								Type:    appsv1.DeploymentAvailable,
@@ -732,6 +783,7 @@ func TestEvaluate(t *testing.T) {
 						},
 					},
 					Status: appsv1.DeploymentStatus{
+						AvailableReplicas: 1,
 						Conditions: []appsv1.DeploymentCondition{
 							{
 								Type:    appsv1.DeploymentAvailable,
@@ -779,6 +831,12 @@ func TestEvaluate(t *testing.T) {
 					Reason:  qv1.ConditionReasonComponentReady,
 					Message: "Clair component healthy",
 				},
+				{
+					Type:    qv1.ComponentClairPostgresReady,
+					Status:  metav1.ConditionTrue,
+					Reason:  qv1.ConditionReasonComponentReady,
+					Message: "ClairPostgres component healthy",
+				},
 				{
 					Type:    qv1.ComponentTLSReady,
 					Status:  metav1.ConditionTrue,
@@ -792,10 +850,10 @@ func TestEvaluate(t *testing.T) {
 					Message: "Deployment registry-quay-redis healthy",
 				},
 				{
-					Type:    qv1.ComponentBaseReady,
+					Type:    qv1.ComponentQuayReady,
 					Status:  metav1.ConditionTrue,
 					Reason:  qv1.ConditionReasonComponentReady,
-					Message: "Base component healthy",
+					Message: "Quay component healthy",
 				},
 				{
 					Type:    qv1.ComponentMirrorReady,
@@ -827,6 +885,10 @@ func TestEvaluate(t *testing.T) {
 							Kind:    qv1.ComponentClair,
 							Managed: true,
 						},
+						{
+							Kind:    qv1.ComponentClairPostgres,
+							Managed: true,
+						},
 						{
 							Kind:    qv1.ComponentRedis,
 							Managed: true,
@@ -854,7 +916,15 @@ func TestEvaluate(t *testing.T) {
 					},
 				},
 			},
-			objs: []runtime.Object{
+			objs: []client.Object{
+				&batchv1.Job{
+					ObjectMeta: metav1.ObjectMeta{
+						Name: "registry-quay-app-upgrade",
+					},
+					Status: batchv1.JobStatus{
+						Succeeded: 1,
+					},
+				},
 				&routev1.Route{
 					ObjectMeta: metav1.ObjectMeta{
 						OwnerReferences: []metav1.OwnerReference{
@@ -957,6 +1027,7 @@ func TestEvaluate(t *testing.T) {
 						},
 					},
 					Status: appsv1.DeploymentStatus{
+						AvailableReplicas: 1,
 						Conditions: []appsv1.DeploymentCondition{
 							{
 								Type:    appsv1.DeploymentAvailable,
@@ -994,6 +1065,7 @@ func TestEvaluate(t *testing.T) {
 						},
 					},
 					Status: appsv1.DeploymentStatus{
+						AvailableReplicas: 1,
 						Conditions: []appsv1.DeploymentCondition{
 							{
 								Type:    appsv1.DeploymentAvailable,
@@ -1016,6 +1088,7 @@ func TestEvaluate(t *testing.T) {
 						},
 					},
 					Status: appsv1.DeploymentStatus{
+						AvailableReplicas: 1,
 						Conditions: []appsv1.DeploymentCondition{
 							{
 								Type:    appsv1.DeploymentAvailable,
@@ -1043,6 +1116,7 @@ func TestEvaluate(t *testing.T) {
 						},
 					},
 					Status: appsv1.DeploymentStatus{
+						AvailableReplicas: 1,
 						Conditions: []appsv1.DeploymentCondition{
 							{
 								Type:    appsv1.DeploymentAvailable,
@@ -1065,6 +1139,7 @@ func TestEvaluate(t *testing.T) {
 						},
 					},
 					Status: appsv1.DeploymentStatus{
+						AvailableReplicas: 1,
 						Conditions: []appsv1.DeploymentCondition{
 							{
 								Type:    appsv1.DeploymentAvailable,
@@ -1087,6 +1162,7 @@ func TestEvaluate(t *testing.T) {
 						},
 					},
 					Status: appsv1.DeploymentStatus{
+						AvailableReplicas: 1,
 						Conditions: []appsv1.DeploymentCondition{
 							{
 								Type:    appsv1.DeploymentAvailable,
@@ -1109,6 +1185,7 @@ func TestEvaluate(t *testing.T) {
 						},
 					},
 					Status: appsv1.DeploymentStatus{
+						AvailableReplicas: 1,
 						Conditions: []appsv1.DeploymentCondition{
 							{
 								Type:    appsv1.DeploymentAvailable,
@@ -1156,6 +1233,12 @@ func TestEvaluate(t *testing.T) {
 					Reason:  qv1.ConditionReasonComponentReady,
 					Message: "Clair component healthy",
 				},
+				{
+					Type:    qv1.ComponentClairPostgresReady,
+					Status:  metav1.ConditionTrue,
+					Reason:  qv1.ConditionReasonComponentReady,
+					Message: "ClairPostgres component healthy",
+				},
 				{
 					Type:    qv1.ComponentTLSReady,
 					Status:  metav1.ConditionTrue,
@@ -1169,10 +1252,10 @@ func TestEvaluate(t *testing.T) {
 					Message: "Deployment registry-quay-redis healthy",
 				},
 				{
-					Type:    qv1.ComponentBaseReady,
+					Type:    qv1.ComponentQuayReady,
 					Status:  metav1.ConditionTrue,
 					Reason:  qv1.ConditionReasonComponentReady,
-					Message: "Base component healthy",
+					Message: "Quay component healthy",
 				},
 				{
 					Type:    qv1.ComponentMirrorReady,
@@ -1206,8 +1289,12 @@ func TestEvaluate(t *testing.T) {
 			if err := monv1.AddToScheme(scheme); err != nil {
 				t.Fatalf("unexpected error adding monitoring to scheme: %s", err)
 			}
+			if err := batchv1.AddToScheme(scheme); err != nil {
+				t.Fatalf("unexpected error adding batch to scheme: %s", err)
+			}
 
-			cli := fake.NewFakeClientWithScheme(scheme, tt.objs...)
+			builder := fake.NewClientBuilder()
+			cli := builder.WithObjects(tt.objs...).WithScheme(scheme).Build()
 			conds, err := Evaluate(ctx, cli, tt.quay)
 			if err != nil {
 				t.Fatalf("unexpected error: %s", err)
