@@ -10,10 +10,11 @@ import (
 	"sort"
 	"strings"
 
-	prometheusv1 "github.com/coreos/prometheus-operator/pkg/apis/monitoring/v1"
 	"github.com/go-logr/logr"
 	objectbucket "github.com/kube-object-storage/lib-bucket-provisioner/pkg/apis/objectbucket.io/v1alpha1"
 	route "github.com/openshift/api/route/v1"
+	prometheusv1 "github.com/prometheus-operator/prometheus-operator/pkg/apis/monitoring/v1"
+	"golang.org/x/net/http/httpproxy"
 	apps "k8s.io/api/apps/v1"
 	autoscaling "k8s.io/api/autoscaling/v2beta2"
 	batchv1 "k8s.io/api/batch/v1"
@@ -23,10 +24,10 @@ import (
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/util/json"
 	"sigs.k8s.io/controller-runtime/pkg/client"
-	"sigs.k8s.io/kustomize/api/filesys"
 	"sigs.k8s.io/kustomize/api/krusty"
-	"sigs.k8s.io/kustomize/api/resid"
 	"sigs.k8s.io/kustomize/api/types"
+	"sigs.k8s.io/kustomize/kyaml/filesys"
+	"sigs.k8s.io/kustomize/kyaml/resid"
 	"sigs.k8s.io/yaml"
 
 	v1 "github.com/quay/quay-operator/apis/quay/v1"
@@ -37,6 +38,7 @@ import (
 const (
 	QuayRegistryNameLabel = "quay-operator/quayregistry"
 
+	proxyConfigPrefix                 = "quay-proxy-config"
 	configSecretPrefix                = "quay-config-secret"
 	registryHostnameAnnotation        = "quay-registry-hostname"
 	buildManagerHostnameAnnotation    = "quay-buildmanager-hostname"
@@ -53,13 +55,13 @@ const (
 // to use. If set, returns a Kustomize image override for the given component.
 func componentImageFor(component v1.ComponentKind) (types.Image, error) {
 	envVarFor := map[v1.ComponentKind]string{
-		v1.ComponentBase:     componentImagePrefix + "QUAY",
+		v1.ComponentQuay:     componentImagePrefix + "QUAY",
 		v1.ComponentClair:    componentImagePrefix + "CLAIR",
 		v1.ComponentRedis:    componentImagePrefix + "REDIS",
 		v1.ComponentPostgres: componentImagePrefix + "POSTGRES",
 	}
 	defaultImagesFor := map[v1.ComponentKind]string{
-		v1.ComponentBase:     "quay.io/projectquay/quay",
+		v1.ComponentQuay:     "quay.io/projectquay/quay",
 		v1.ComponentClair:    "quay.io/projectquay/clair",
 		v1.ComponentRedis:    "centos/redis-32-centos7",
 		v1.ComponentPostgres: "centos/postgresql-10-centos7",
@@ -68,17 +70,22 @@ func componentImageFor(component v1.ComponentKind) (types.Image, error) {
 	imageOverride := types.Image{
 		Name: defaultImagesFor[component],
 	}
+
 	image := os.Getenv(envVarFor[component])
-	if image != "" {
-		if len(strings.Split(image, "@")) == 2 {
-			imageOverride.NewName = strings.Split(image, "@")[0]
-			imageOverride.Digest = strings.Split(image, "@")[1]
-		} else if len(strings.Split(image, ":")) == 2 {
-			imageOverride.NewName = strings.Split(image, ":")[0]
-			imageOverride.NewTag = strings.Split(image, ":")[1]
-		} else {
-			return types.Image{}, errors.New("Image override must be reference by tag or manifest digest: " + image)
-		}
+	if image == "" {
+		return imageOverride, nil
+	}
+
+	if len(strings.Split(image, "@")) == 2 {
+		imageOverride.NewName = strings.Split(image, "@")[0]
+		imageOverride.Digest = strings.Split(image, "@")[1]
+	} else if len(strings.Split(image, ":")) == 2 {
+		imageOverride.NewName = strings.Split(image, ":")[0]
+		imageOverride.NewTag = strings.Split(image, ":")[1]
+	} else {
+		return types.Image{}, fmt.Errorf(
+			"image override must be reference by tag or digest: %s", image,
+		)
 	}
 
 	return imageOverride, nil
@@ -87,7 +94,6 @@ func componentImageFor(component v1.ComponentKind) (types.Image, error) {
 func kustomizeDir() string {
 	_, filename, _, _ := runtime.Caller(0)
 	path := filepath.Join(filepath.Dir(filename))
-
 	return filepath.Join(path, "..", "..", "kustomize")
 }
 
@@ -118,32 +124,27 @@ func rolloutBlocked(quay *v1.QuayRegistry) bool {
 	return false
 }
 
-func check(err error) {
-	if err != nil {
-		panic(err)
-	}
-}
-
 func encode(value interface{}) []byte {
 	yamlified, _ := yaml.Marshal(value)
-
 	return yamlified
 }
 
 func decode(bytes []byte) interface{} {
 	var value interface{}
 	_ = yaml.Unmarshal(bytes, &value)
-
 	return value
 }
 
-// EnsureCreationOrder sorts the given slice of Kubernetes objects so that
-// when created in order, `Deployments` will be created after their dependencies (`Secrets`/`ConfigMaps`/etc).
+// EnsureCreationOrder sorts the given slice of Kubernetes objects so that when created in order,
+// `Deployments` will be created after their dependencies (`Secrets`/`ConfigMaps`/etc).
 func EnsureCreationOrder(objects []client.Object) []client.Object {
-	sort.Slice(objects, func(i, j int) bool {
-		return objects[i].GetObjectKind().GroupVersionKind().String() != schema.GroupVersionKind{Group: "apps", Version: "v1", Kind: "Deployment"}.String()
-	})
-
+	sort.Slice(
+		objects,
+		func(i, j int) bool {
+			obji := objects[i]
+			return obji.GetObjectKind().GroupVersionKind().Kind != "Deployment"
+		},
+	)
 	return objects
 }
 
@@ -187,63 +188,83 @@ func ModelFor(gvk schema.GroupVersionKind) client.Object {
 }
 
 // generate uses Kustomize as a library to build the runtime objects to be applied to a cluster.
-func generate(kustomization *types.Kustomization, overlay string, quayConfigFiles map[string][]byte) ([]client.Object, error) {
+func generate(
+	kustomization *types.Kustomization, overlay string, quayConfigFiles map[string][]byte,
+) ([]client.Object, error) {
 	fSys := filesys.MakeEmptyDirInMemory()
-	err := filepath.Walk(kustomizeDir(), func(path string, info os.FileInfo, err error) error {
-		if err != nil {
-			return err
-		}
+	if err := filepath.Walk(
+		kustomizeDir(),
+		func(path string, info os.FileInfo, err error) error {
+			if err != nil {
+				return err
+			}
+
+			if info.IsDir() {
+				return nil
+			}
 
-		if !info.IsDir() {
 			f, err := ioutil.ReadFile(path)
 			if err != nil {
 				return err
 			}
 
-			err = fSys.WriteFile(path, f)
-			if err != nil {
-				return err
-			}
-		}
-		return nil
-	})
-	check(err)
+			return fSys.WriteFile(path, f)
+		},
+	); err != nil {
+		return nil, err
+	}
 
 	// Write `kustomization.yaml` to filesystem
 	kustomizationFile, err := yaml.Marshal(kustomization)
-	check(err)
-	err = fSys.WriteFile(filepath.Join(appDir(), "kustomization.yaml"), kustomizationFile)
-	check(err)
+	if err != nil {
+		return nil, err
+	}
+
+	if err := fSys.WriteFile(
+		filepath.Join(appDir(), "kustomization.yaml"), kustomizationFile,
+	); err != nil {
+		return nil, err
+	}
 
 	// Add all Quay config files to directory to be included in the generated `Secret`
 	for fileName, file := range quayConfigFiles {
-		check(err)
-		err = fSys.WriteFile(filepath.Join(appDir(), "bundle", fileName), file)
-		check(err)
+		if err := fSys.WriteFile(
+			filepath.Join(appDir(), "bundle", fileName), file,
+		); err != nil {
+			return nil, err
+		}
 	}
 
-	opts := &krusty.Options{}
-	k := krusty.MakeKustomizer(fSys, opts)
-	resMap, err := k.Run(overlay)
-	check(err)
+	opts := &krusty.Options{
+		PluginConfig: &types.PluginConfig{},
+	}
+	k := krusty.MakeKustomizer(opts)
+	resMap, err := k.Run(fSys, overlay)
+	if err != nil {
+		return nil, err
+	}
 
 	output := []client.Object{}
 	for _, resource := range resMap.Resources() {
 		resourceJSON, err := resource.MarshalJSON()
-		check(err)
+		if err != nil {
+			return nil, err
+		}
 
-		obj := ModelFor(schema.GroupVersionKind{
+		gvk := schema.GroupVersionKind{
 			Group:   resource.GetGvk().Group,
 			Version: resource.GetGvk().Version,
 			Kind:    resource.GetGvk().Kind,
-		})
-
-		if obj == nil {
-			panic("TODO: Not implemented for GroupVersionKind: " + resource.GetGvk().String())
 		}
 
-		err = json.Unmarshal(resourceJSON, obj)
-		check(err)
+		obj := ModelFor(gvk)
+		if obj == nil {
+			return nil, fmt.Errorf("kind not supported: %s", gvk.Kind)
+		}
+
+		if err := json.Unmarshal(resourceJSON, obj); err != nil {
+			return nil, err
+		}
 
 		output = append(output, obj)
 	}
@@ -252,7 +273,13 @@ func generate(kustomization *types.Kustomization, overlay string, quayConfigFile
 }
 
 // KustomizationFor takes a `QuayRegistry` object and generates a Kustomization for it.
-func KustomizationFor(ctx *quaycontext.QuayRegistryContext, quay *v1.QuayRegistry, quayConfigFiles map[string][]byte) (*types.Kustomization, error) {
+func KustomizationFor(
+	log logr.Logger,
+	ctx *quaycontext.QuayRegistryContext,
+	quay *v1.QuayRegistry,
+	quayConfigFiles map[string][]byte,
+	overlay string,
+) (*types.Kustomization, error) {
 	if quay == nil {
 		return nil, errors.New("given QuayRegistry should not be nil")
 	}
@@ -262,9 +289,11 @@ func KustomizationFor(ctx *quaycontext.QuayRegistryContext, quay *v1.QuayRegistr
 		configFiles = append(configFiles, filepath.Join("bundle", key))
 	}
 
-	configEditorPassword, err := generateRandomString(16)
-	if err != nil {
-		return nil, err
+	if len(ctx.ConfigEditorPw) == 0 {
+		var err error
+		if ctx.ConfigEditorPw, err = generateRandomString(16); err != nil {
+			return nil, err
+		}
 	}
 
 	if ctx.DbRootPw == "" {
@@ -275,6 +304,14 @@ func KustomizationFor(ctx *quaycontext.QuayRegistryContext, quay *v1.QuayRegistr
 		ctx.DbRootPw = rootpw
 	}
 
+	userProvidedCaCerts := []string{}
+	for key, val := range quayConfigFiles {
+		if !strings.HasPrefix(key, "extra_ca_cert_") {
+			continue
+		}
+		userProvidedCaCerts = append(userProvidedCaCerts, strings.TrimPrefix(key, "extra_ca_cert_")+"="+string(val))
+	}
+
 	quayConfigTLSSources := []string{}
 	if ctx.ClusterWildcardCert != nil {
 		quayConfigTLSSources = append(quayConfigTLSSources, "ocp-cluster-wildcard.cert="+string(ctx.ClusterWildcardCert))
@@ -286,7 +323,28 @@ func KustomizationFor(ctx *quaycontext.QuayRegistryContext, quay *v1.QuayRegistr
 		quayConfigTLSSources = append(quayConfigTLSSources, "ssl.key="+string(ctx.TLSKey))
 	}
 
+	// read the proxy configuration from the environment. makes sure we add the quay server
+	// host name to the list of addresses reachable without proxy (NO_PROXY).
+	proxyenv := httpproxy.FromEnvironment()
+	addrs := []string{ctx.ServerHostname}
+	if len(proxyenv.NoProxy) > 0 {
+		noproxy := strings.Split(proxyenv.NoProxy, ",")
+		addrs = append(addrs, noproxy...)
+	}
+
 	generatedSecrets := []types.SecretArgs{
+		{
+			GeneratorArgs: types.GeneratorArgs{
+				Name: proxyConfigPrefix,
+				KvPairSources: types.KvPairSources{
+					LiteralSources: []string{
+						"HTTP_PROXY=" + proxyenv.HTTPProxy,
+						"HTTPS_PROXY=" + proxyenv.HTTPSProxy,
+						"NO_PROXY=" + strings.Join(addrs, ","),
+					},
+				},
+			},
+		},
 		{
 			GeneratorArgs: types.GeneratorArgs{
 				Name: configSecretPrefix,
@@ -295,15 +353,28 @@ func KustomizationFor(ctx *quaycontext.QuayRegistryContext, quay *v1.QuayRegistr
 				},
 			},
 		},
+		{
+			GeneratorArgs: types.GeneratorArgs{
+				Name: "extra-ca-certs",
+				KvPairSources: types.KvPairSources{
+					LiteralSources: userProvidedCaCerts,
+				},
+			},
+		},
 		{
 			GeneratorArgs: types.GeneratorArgs{
 				Name: v1.ManagedKeysName,
+				Options: &types.GeneratorOptions{
+					DisableNameSuffixHash: true,
+				},
 				KvPairSources: types.KvPairSources{
 					LiteralSources: []string{
 						"DATABASE_SECRET_KEY=" + ctx.DatabaseSecretKey,
 						"SECRET_KEY=" + ctx.SecretKey,
 						"DB_URI=" + ctx.DbUri,
 						"DB_ROOT_PW=" + ctx.DbRootPw,
+						"CONFIG_EDITOR_PW=" + ctx.ConfigEditorPw,
+						"SECURITY_SCANNER_V4_PSK=" + ctx.SecurityScannerV4PSK,
 					},
 				},
 			},
@@ -322,7 +393,7 @@ func KustomizationFor(ctx *quaycontext.QuayRegistryContext, quay *v1.QuayRegistr
 				KvPairSources: types.KvPairSources{
 					LiteralSources: []string{
 						"username=" + configEditorUser,
-						"password=" + configEditorPassword,
+						"password=" + ctx.ConfigEditorPw,
 					},
 				},
 			},
@@ -330,21 +401,34 @@ func KustomizationFor(ctx *quaycontext.QuayRegistryContext, quay *v1.QuayRegistr
 	}
 
 	componentPaths := []string{}
+	if overlay == upgradeOverlayDir() {
+		componentPaths = []string{"../components/job"}
+	}
+
 	for _, component := range quay.Spec.Components {
-		if component.Managed {
-			componentPaths = append(componentPaths, filepath.Join("..", "components", string(component.Kind)))
+		if !component.Managed || component.Kind == v1.ComponentQuay {
+			continue
+		}
 
-			componentConfigFiles, err := componentConfigFilesFor(ctx, component.Kind, quay, quayConfigFiles)
-			if componentConfigFiles == nil || err != nil {
-				continue
-			}
+		componentPaths = append(
+			componentPaths, filepath.Join("..", "components", string(component.Kind)),
+		)
 
-			sources := []string{}
-			for filename, fileValue := range componentConfigFiles {
-				sources = append(sources, strings.Join([]string{filename, string(fileValue)}, "="))
-			}
+		componentConfigFiles, err := componentConfigFilesFor(
+			log, ctx, component.Kind, quay, quayConfigFiles,
+		)
+		if componentConfigFiles == nil || err != nil {
+			continue
+		}
 
-			generatedSecrets = append(generatedSecrets, types.SecretArgs{
+		sources := []string{}
+		for filename, fileValue := range componentConfigFiles {
+			sources = append(sources, strings.Join([]string{filename, string(fileValue)}, "="))
+		}
+
+		generatedSecrets = append(
+			generatedSecrets,
+			types.SecretArgs{
 				GeneratorArgs: types.GeneratorArgs{
 					Name:     string(component.Kind) + "-config-secret",
 					Behavior: "merge",
@@ -352,12 +436,14 @@ func KustomizationFor(ctx *quaycontext.QuayRegistryContext, quay *v1.QuayRegistr
 						LiteralSources: sources,
 					},
 				},
-			})
-		}
+			},
+		)
 	}
 
 	images := []types.Image{}
-	for _, component := range append(quay.Spec.Components, v1.Component{Kind: "base", Managed: true}) {
+	for _, component := range append(
+		quay.Spec.Components, v1.Component{Kind: "quay", Managed: true},
+	) {
 		image, err := componentImageFor(component.Kind)
 		if err != nil {
 			return nil, err
@@ -402,30 +488,32 @@ func KustomizationFor(ctx *quaycontext.QuayRegistryContext, quay *v1.QuayRegistr
 	}, nil
 }
 
-// Inflate takes a `QuayRegistry` object and returns a set of Kubernetes objects representing a Quay deployment.
+// Inflate takes a `QuayRegistry` object and returns a set of Kubernetes objects representing a
+// Quay deployment.
 func Inflate(
 	ctx *quaycontext.QuayRegistryContext,
 	quay *v1.QuayRegistry,
-	baseConfigBundle *corev1.Secret,
+	bundle *corev1.Secret,
 	log logr.Logger,
+	skipres bool,
 ) ([]client.Object, error) {
-	// Each managed component brings its own generated `config.yaml` fields
-	// which are accumulated under the key `<component>.config.yaml` and then added to the base `Secret`.
-	componentConfigFiles := baseConfigBundle.DeepCopy().Data
+	// Each managed component brings its own generated `config.yaml` fields which are
+	// accumulated under the key <component>.config.yaml and then added to the base `Secret`.
+	componentConfigFiles := bundle.DeepCopy().Data
 
 	var parsedUserConfig map[string]interface{}
-	if err := yaml.Unmarshal(baseConfigBundle.Data["config.yaml"], &parsedUserConfig); err != nil {
+	if err := yaml.Unmarshal(bundle.Data["config.yaml"], &parsedUserConfig); err != nil {
 		return nil, err
 	}
 
-	// Generate or pull out the `SECRET_KEY` and `DATABASE_SECRET_KEY`. Since these must be stable across
-	// runs of the same config, we store them (and re-read them) from a specialized `Secret`.
-	// TODO(alecmerdler): Refector these three blocks...
-	if key, found := parsedUserConfig["DATABASE_SECRET_KEY"].(string); found && len(key) > 0 {
+	// Generate or pull out the `SECRET_KEY` and `DATABASE_SECRET_KEY`. Since these must be
+	// stable across runs of the same config, we store them (and re-read them) from a
+	// specialized `Secret`.  TODO(alecmerdler): Refector these three blocks...
+	if key, ok := parsedUserConfig["DATABASE_SECRET_KEY"].(string); ok && len(key) > 0 {
 		log.Info("`DATABASE_SECRET_KEY` found in user-provided config")
 		ctx.DatabaseSecretKey = key
 	} else if ctx.DatabaseSecretKey == "" {
-		log.Info("`DATABASE_SECRET_KEY` not found in user-provided config, generating new one")
+		log.Info("`DATABASE_SECRET_KEY` not found in user-provided config, generating")
 		databaseSecretKey, err := generateRandomString(secretKeyLength)
 		if err != nil {
 			return nil, err
@@ -434,11 +522,11 @@ func Inflate(
 	}
 	parsedUserConfig["DATABASE_SECRET_KEY"] = ctx.DatabaseSecretKey
 
-	if key, found := parsedUserConfig["SECRET_KEY"].(string); found && len(key) > 0 {
+	if key, ok := parsedUserConfig["SECRET_KEY"].(string); ok && len(key) > 0 {
 		log.Info("`SECRET_KEY` found in user-provided config")
 		ctx.SecretKey = key
 	} else if ctx.SecretKey == "" {
-		log.Info("`SECRET_KEY` not found in user-provided config, generating new one")
+		log.Info("`SECRET_KEY` not found in user-provided config, generating")
 		secretKey, err := generateRandomString(secretKeyLength)
 		if err != nil {
 			return nil, err
@@ -447,9 +535,12 @@ func Inflate(
 	}
 	parsedUserConfig["SECRET_KEY"] = ctx.SecretKey
 
-	if dbURI, found := parsedUserConfig["DB_URI"].(string); found && len(dbURI) > 0 {
+	var dbCfgHasChanged bool
+	if dbURI, ok := parsedUserConfig["DB_URI"].(string); ok && len(dbURI) > 0 {
+		dbCfgHasChanged = parsedUserConfig["DB_URI"] != ctx.DbUri
 		ctx.DbUri = dbURI
 	} else if v1.ComponentIsManaged(quay.Spec.Components, v1.ComponentPostgres) && len(ctx.DbUri) == 0 {
+		dbCfgHasChanged = true
 		log.Info("managed `DB_URI` not found in config, generating new one")
 		user := quay.GetName() + "-quay-database"
 		name := quay.GetName() + "-quay-database"
@@ -465,7 +556,7 @@ func Inflate(
 	}
 	parsedUserConfig["DB_URI"] = ctx.DbUri
 
-	for field, value := range BaseConfig() {
+	for field, value := range BaseQuayConfig() {
 		if _, ok := parsedUserConfig[field]; !ok {
 			parsedUserConfig[field] = value
 		}
@@ -473,14 +564,17 @@ func Inflate(
 	componentConfigFiles["config.yaml"] = encode(parsedUserConfig)
 
 	for _, component := range quay.Spec.Components {
-		if component.Managed {
-			fieldGroup, err := FieldGroupFor(ctx, component.Kind, quay)
-			if err != nil {
-				return nil, err
-			}
-
-			componentConfigFiles[string(component.Kind)+".config.yaml"] = encode(fieldGroup)
+		if !component.Managed {
+			continue
 		}
+
+		fieldGroup, err := FieldGroupFor(ctx, component.Kind, quay)
+		if err != nil {
+			return nil, err
+		}
+
+		index := fmt.Sprintf("%s.config.yaml", component.Kind)
+		componentConfigFiles[index] = encode(fieldGroup)
 	}
 
 	log.Info("Ensuring TLS cert/key pair for Quay app")
@@ -488,19 +582,16 @@ func Inflate(
 	if err != nil {
 		return nil, err
 	}
-
 	ctx.TLSCert = tlsCert
 	ctx.TLSKey = tlsKey
 
-	kustomization, err := KustomizationFor(ctx, quay, componentConfigFiles)
-	if err != nil {
-		return nil, err
-	}
-
 	var overlay string
 	if rolloutBlocked(quay) {
 		overlay = configEditorOnlyOverlay()
-	} else if quay.Status.CurrentVersion != v1.QuayVersionCurrent {
+	} else if quay.Status.CurrentVersion != v1.QuayVersionCurrent || dbCfgHasChanged {
+		// we render the upgrade overlay directory only if the operator version or the
+		// database configuration has changed. this scales down quay and runs a job to
+		// migrate the database.
 		overlay = upgradeOverlayDir()
 	} else if !v1.ComponentIsManaged(quay.Spec.Components, v1.ComponentTLS) {
 		overlay = unmanagedTLSOverlayDir()
@@ -508,13 +599,18 @@ func Inflate(
 		overlay = overlayDir()
 	}
 
+	kustomization, err := KustomizationFor(log, ctx, quay, componentConfigFiles, overlay)
+	if err != nil {
+		return nil, err
+	}
+
 	resources, err := generate(kustomization, overlay, componentConfigFiles)
 	if err != nil {
 		return nil, err
 	}
 
 	for index, resource := range resources {
-		obj, err := middleware.Process(ctx, quay, resource)
+		obj, err := middleware.Process(quay, resource, skipres)
 		if err != nil {
 			return nil, err
 		}
@@ -523,12 +619,7 @@ func Inflate(
 	}
 
 	for index, resource := range resources {
-		resource, err = v1.EnsureOwnerReference(quay, resource)
-		if err != nil {
-			return nil, err
-		}
-
-		resources[index] = resource
+		resources[index] = v1.EnsureOwnerReference(quay, resource)
 	}
 
 	return resources, err
@@ -542,7 +633,7 @@ func operatorServiceEndpoint() string {
 
 	endpoint := "http://quay-operator"
 	if ns := os.Getenv(podNamespaceKey); ns != "" {
-		endpoint = strings.Join([]string{endpoint, ns}, ".")
+		endpoint = fmt.Sprintf("http://quay-operator.%s.svc.cluster.local", ns)
 	}
 
 	return strings.Join([]string{endpoint, "7071"}, ":")
