@@ -5,6 +5,7 @@ import (
 	"strings"
 	"testing"
 
+	"github.com/go-logr/logr"
 	testlogr "github.com/go-logr/logr/testing"
 	objectbucket "github.com/kube-object-storage/lib-bucket-provisioner/pkg/apis/objectbucket.io/v1alpha1"
 	"github.com/stretchr/testify/assert"
@@ -169,6 +170,7 @@ var kustomizationForTests = []struct {
 
 func TestKustomizationFor(t *testing.T) {
 	assert := assert.New(t)
+	log := logr.Discard()
 
 	for _, test := range kustomizationForTests {
 		if test.expected != nil {
@@ -181,7 +183,7 @@ func TestKustomizationFor(t *testing.T) {
 			}
 		}
 
-		kustomization, err := KustomizationFor(&test.ctx, test.quayRegistry, map[string][]byte{})
+		kustomization, err := KustomizationFor(log, &test.ctx, test.quayRegistry, map[string][]byte{}, "")
 
 		if test.expectedErr != "" {
 			assert.EqualError(err, test.expectedErr)
@@ -237,9 +239,8 @@ func TestFlattenSecret(t *testing.T) {
 }
 
 var quayComponents = map[string][]client.Object{
-	"base": {
+	"quay": {
 		&appsv1.Deployment{ObjectMeta: metav1.ObjectMeta{Name: "quay-app"}},
-		&batchv1.Job{ObjectMeta: metav1.ObjectMeta{Name: "quay-app-upgrade"}},
 		&appsv1.Deployment{ObjectMeta: metav1.ObjectMeta{Name: "quay-config-editor"}},
 		&corev1.Service{ObjectMeta: metav1.ObjectMeta{Name: "quay-app"}},
 		&corev1.Service{ObjectMeta: metav1.ObjectMeta{Name: "quay-config-editor"}},
@@ -249,6 +250,9 @@ var quayComponents = map[string][]client.Object{
 		&corev1.Secret{ObjectMeta: metav1.ObjectMeta{Name: "quay-config-editor-credentials"}},
 		&corev1.Secret{ObjectMeta: metav1.ObjectMeta{Name: "quay-registry-managed-secret-keys"}},
 		&corev1.ServiceAccount{ObjectMeta: metav1.ObjectMeta{Name: "quay-app"}},
+		&corev1.ConfigMap{ObjectMeta: metav1.ObjectMeta{Name: "cluster-service-ca"}},
+		&corev1.ConfigMap{ObjectMeta: metav1.ObjectMeta{Name: "extra-ca-certs"}},
+		&corev1.Secret{ObjectMeta: metav1.ObjectMeta{Name: "quay-proxy-config"}},
 	},
 	"clair": {
 		&corev1.Secret{ObjectMeta: metav1.ObjectMeta{Name: "clair-config-secret"}},
@@ -267,7 +271,6 @@ var quayComponents = map[string][]client.Object{
 		&appsv1.Deployment{ObjectMeta: metav1.ObjectMeta{Name: "quay-database"}},
 		&corev1.PersistentVolumeClaim{ObjectMeta: metav1.ObjectMeta{Name: "quay-database"}},
 		&corev1.Service{ObjectMeta: metav1.ObjectMeta{Name: "quay-database"}},
-		&batchv1.Job{ObjectMeta: metav1.ObjectMeta{Name: "quay-database-init"}},
 		&corev1.ServiceAccount{ObjectMeta: metav1.ObjectMeta{Name: "quay-database"}},
 		&corev1.ConfigMap{ObjectMeta: metav1.ObjectMeta{Name: "postgres-conf-sample"}},
 	},
@@ -290,6 +293,9 @@ var quayComponents = map[string][]client.Object{
 		&autoscaling.HorizontalPodAutoscaler{ObjectMeta: metav1.ObjectMeta{Name: "quay-mirror"}},
 		&autoscaling.HorizontalPodAutoscaler{ObjectMeta: metav1.ObjectMeta{Name: "clair-app"}},
 	},
+	"job": {
+		&batchv1.Job{ObjectMeta: metav1.ObjectMeta{Name: "quay-app-upgrade"}},
+	},
 }
 
 func withComponents(components []string) []client.Object {
@@ -317,6 +323,7 @@ var inflateTests = []struct {
 				Components: []v1.Component{
 					{Kind: "postgres", Managed: true},
 					{Kind: "clair", Managed: true},
+					{Kind: "clairpostgres", Managed: true},
 					{Kind: "redis", Managed: true},
 					{Kind: "objectstorage", Managed: true},
 					{Kind: "mirror", Managed: true},
@@ -332,7 +339,7 @@ var inflateTests = []struct {
 				"config.yaml": encode(map[string]interface{}{"SERVER_HOSTNAME": "quay.io"}),
 			},
 		},
-		expected:    withComponents([]string{"base", "clair", "postgres", "redis", "objectstorage", "mirror", "horizontalpodautoscaler"}),
+		expected:    withComponents([]string{"job", "quay", "clair", "postgres", "redis", "objectstorage", "mirror", "horizontalpodautoscaler", "clairpostgres"}),
 		expectedErr: nil,
 	},
 	{
@@ -342,6 +349,7 @@ var inflateTests = []struct {
 				Components: []v1.Component{
 					{Kind: "postgres", Managed: false},
 					{Kind: "clair", Managed: false},
+					{Kind: "clairpostgres", Managed: false},
 					{Kind: "redis", Managed: false},
 					{Kind: "objectstorage", Managed: false},
 					{Kind: "mirror", Managed: false},
@@ -355,7 +363,7 @@ var inflateTests = []struct {
 				"config.yaml": encode(map[string]interface{}{"SERVER_HOSTNAME": "quay.io"}),
 			},
 		},
-		expected:    withComponents([]string{"base"}),
+		expected:    withComponents([]string{"quay"}),
 		expectedErr: nil,
 	},
 	{
@@ -365,6 +373,7 @@ var inflateTests = []struct {
 				Components: []v1.Component{
 					{Kind: "postgres", Managed: true},
 					{Kind: "clair", Managed: true},
+					{Kind: "clairpostgres", Managed: true},
 					{Kind: "redis", Managed: false},
 					{Kind: "objectstorage", Managed: false},
 					{Kind: "mirror", Managed: true},
@@ -377,7 +386,7 @@ var inflateTests = []struct {
 				"config.yaml": encode(map[string]interface{}{"SERVER_HOSTNAME": "quay.io"}),
 			},
 		},
-		expected:    withComponents([]string{"base", "postgres", "clair", "mirror"}),
+		expected:    withComponents([]string{"job", "quay", "postgres", "clair", "mirror", "clairpostgres"}),
 		expectedErr: nil,
 	},
 	{
@@ -387,6 +396,7 @@ var inflateTests = []struct {
 				Components: []v1.Component{
 					{Kind: "postgres", Managed: true},
 					{Kind: "clair", Managed: true},
+					{Kind: "clairpostgres", Managed: true},
 					{Kind: "redis", Managed: true},
 					{Kind: "objectstorage", Managed: true},
 					{Kind: "mirror", Managed: true},
@@ -398,13 +408,14 @@ var inflateTests = []struct {
 		},
 		ctx: quaycontext.QuayRegistryContext{
 			SupportsObjectStorage: true,
+			DbUri:                 "postgresql://user:pass@db:5432/db",
 		},
 		configBundle: &corev1.Secret{
 			Data: map[string][]byte{
 				"config.yaml": encode(map[string]interface{}{"SERVER_HOSTNAME": "quay.io"}),
 			},
 		},
-		expected:    withComponents([]string{"base", "clair", "postgres", "redis", "objectstorage", "mirror"}),
+		expected:    withComponents([]string{"quay", "clair", "postgres", "redis", "objectstorage", "mirror", "clairpostgres"}),
 		expectedErr: nil,
 	},
 	{
@@ -414,6 +425,7 @@ var inflateTests = []struct {
 				Components: []v1.Component{
 					{Kind: "postgres", Managed: true},
 					{Kind: "clair", Managed: true},
+					{Kind: "clairpostgres", Managed: true},
 					{Kind: "redis", Managed: true},
 					{Kind: "objectstorage", Managed: true},
 					{Kind: "mirror", Managed: true},
@@ -425,13 +437,14 @@ var inflateTests = []struct {
 		},
 		ctx: quaycontext.QuayRegistryContext{
 			SupportsObjectStorage: true,
+			DbUri:                 "postgresql://user:pass@db:5432/db",
 		},
 		configBundle: &corev1.Secret{
 			Data: map[string][]byte{
 				"config.yaml": encode(map[string]interface{}{"SERVER_HOSTNAME": "quay.io", "DATABASE_SECRET_KEY": "abc123"}),
 			},
 		},
-		expected:    withComponents([]string{"base", "clair", "postgres", "redis", "objectstorage", "mirror"}),
+		expected:    withComponents([]string{"quay", "clair", "postgres", "redis", "objectstorage", "mirror", "clairpostgres"}),
 		expectedErr: nil,
 	},
 	{
@@ -451,7 +464,7 @@ var inflateTests = []struct {
 				"config.yaml": encode(map[string]interface{}{"SERVER_HOSTNAME": "quay.io"}),
 			},
 		},
-		expected:    withComponents([]string{"base", "postgres"}),
+		expected:    withComponents([]string{"quay", "postgres"}),
 		expectedErr: nil,
 	},
 	{
@@ -472,7 +485,123 @@ var inflateTests = []struct {
 				}),
 			},
 		},
-		expected:    withComponents([]string{"base"}),
+		expected:    withComponents([]string{"job", "quay"}),
+		expectedErr: nil,
+	},
+	{
+		name: "PostgresConfigurationManuallyUpdated",
+		quayRegistry: &v1.QuayRegistry{
+			Spec: v1.QuayRegistrySpec{
+				Components: []v1.Component{
+					{Kind: "postgres", Managed: false},
+				},
+			},
+			Status: v1.QuayRegistryStatus{
+				CurrentVersion: v1.QuayVersionCurrent,
+			},
+		},
+		ctx: quaycontext.QuayRegistryContext{
+			DbUri: "postgresql://olduser:oldpass@olddb:5432/olddatabase",
+		},
+		configBundle: &corev1.Secret{
+			Data: map[string][]byte{
+				"config.yaml": encode(map[string]interface{}{
+					"SERVER_HOSTNAME": "quay.io",
+					"DB_URI":          "postgresql://test-quay-database:postgres@test-quay-database:5432/test-quay-database",
+				}),
+			},
+		},
+		expected:    withComponents([]string{"job", "quay"}),
+		expectedErr: nil,
+	},
+	{
+		name: "NoChangeInDatabaseButUpgradedVersion",
+		quayRegistry: &v1.QuayRegistry{
+			Spec: v1.QuayRegistrySpec{
+				Components: []v1.Component{
+					{Kind: "postgres", Managed: false},
+				},
+			},
+			Status: v1.QuayRegistryStatus{
+				CurrentVersion: "v0.0.0",
+			},
+		},
+		ctx: quaycontext.QuayRegistryContext{
+			DbUri: "postgresql://olduser:oldpass@olddb:5432/olddatabase",
+		},
+		configBundle: &corev1.Secret{
+			Data: map[string][]byte{
+				"config.yaml": encode(map[string]interface{}{
+					"SERVER_HOSTNAME": "quay.io",
+					"DB_URI":          "postgresql://olduser:oldpass@olddb:5432/olddatabase",
+				}),
+			},
+		},
+		expected:    withComponents([]string{"job", "quay"}),
+		expectedErr: nil,
+	},
+	{
+		name: "RerenderWithoutChanges",
+		quayRegistry: &v1.QuayRegistry{
+			Spec: v1.QuayRegistrySpec{
+				Components: []v1.Component{
+					{Kind: "postgres", Managed: true},
+					{Kind: "clair", Managed: true},
+					{Kind: "clairpostgres", Managed: true},
+					{Kind: "redis", Managed: true},
+					{Kind: "objectstorage", Managed: true},
+					{Kind: "mirror", Managed: true},
+					{Kind: "horizontalpodautoscaler", Managed: true},
+				},
+			},
+		},
+		ctx: quaycontext.QuayRegistryContext{
+			SupportsObjectStorage: true,
+			DbUri:                 "postgresql://user:pass@db:5432/db",
+		},
+		configBundle: &corev1.Secret{
+			Data: map[string][]byte{
+				"config.yaml": encode(
+					map[string]interface{}{
+						"SERVER_HOSTNAME": "quay.io",
+						"DB_URI":          "postgresql://user:pass@db:5432/db",
+					},
+				),
+			},
+		},
+		expected:    withComponents([]string{"quay", "clair", "postgres", "redis", "objectstorage", "mirror", "horizontalpodautoscaler", "clairpostgres"}),
+		expectedErr: nil,
+	},
+	{
+		name: "RerenderWithDatabaseChanges",
+		quayRegistry: &v1.QuayRegistry{
+			Spec: v1.QuayRegistrySpec{
+				Components: []v1.Component{
+					{Kind: "postgres", Managed: true},
+					{Kind: "clair", Managed: true},
+					{Kind: "clairpostgres", Managed: true},
+					{Kind: "redis", Managed: true},
+					{Kind: "objectstorage", Managed: true},
+					{Kind: "mirror", Managed: true},
+					{Kind: "horizontalpodautoscaler", Managed: true},
+				},
+			},
+		},
+		ctx: quaycontext.QuayRegistryContext{
+			SupportsObjectStorage: true,
+			DbUri:                 "postgresql://user:pass@db:5432/db",
+		},
+		configBundle: &corev1.Secret{
+			Data: map[string][]byte{
+				"config.yaml": encode(
+					map[string]interface{}{
+						"SERVER_HOSTNAME": "quay.io",
+						"DB_URI":          "postgresql://new:new@new:5432/db",
+					},
+				),
+			},
+		},
+		expected:    withComponents([]string{"job", "quay", "clair", "postgres", "redis", "objectstorage", "mirror", "horizontalpodautoscaler", "clairpostgres"}),
 		expectedErr: nil,
 	},
 }
@@ -480,11 +609,11 @@ var inflateTests = []struct {
 func TestInflate(t *testing.T) {
 	assert := assert.New(t)
 
-	log := testlogr.TestLogger{}
+	log := testlogr.NewTestLogger(t)
 
 	for _, test := range inflateTests {
 		t.Run(test.name, func(t *testing.T) {
-			pieces, err := Inflate(&test.ctx, test.quayRegistry, test.configBundle, log)
+			pieces, err := Inflate(&test.ctx, test.quayRegistry, test.configBundle, log, false)
 
 			assert.NotNil(pieces, test.name)
 			assert.Equal(len(test.expected), len(pieces), test.name)
