@@ -13,17 +13,10 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/client"
 
 	v1 "github.com/quay/quay-operator/apis/quay/v1"
-	quaycontext "github.com/quay/quay-operator/pkg/context"
-)
-
-const (
-	tlsCert = "my-own-cert"
-	tlsKey  = "my-own-key"
 )
 
 var processTests = []struct {
 	name          string
-	ctx           *quaycontext.QuayRegistryContext
 	quay          *v1.QuayRegistry
 	obj           client.Object
 	expected      client.Object
@@ -31,7 +24,6 @@ var processTests = []struct {
 }{
 	{
 		"quayConfigBundle",
-		&quaycontext.QuayRegistryContext{},
 		&v1.QuayRegistry{},
 		&corev1.Secret{
 			ObjectMeta: metav1.ObjectMeta{
@@ -49,10 +41,6 @@ var processTests = []struct {
 	},
 	{
 		"quayAppRouteTLSUnmanaged",
-		&quaycontext.QuayRegistryContext{
-			TLSCert: []byte(tlsCert),
-			TLSKey:  []byte(tlsKey),
-		},
 		&v1.QuayRegistry{
 			Spec: v1.QuayRegistrySpec{
 				Components: []v1.Component{
@@ -84,10 +72,6 @@ var processTests = []struct {
 	},
 	{
 		"quayBuilderRouteTLSUnmanaged",
-		&quaycontext.QuayRegistryContext{
-			TLSCert: []byte(tlsCert),
-			TLSKey:  []byte(tlsKey),
-		},
 		&v1.QuayRegistry{
 			Spec: v1.QuayRegistrySpec{
 				Components: []v1.Component{
@@ -116,10 +100,6 @@ var processTests = []struct {
 	},
 	{
 		"quayAppRouteTLSManaged",
-		&quaycontext.QuayRegistryContext{
-			TLSCert: []byte(tlsCert),
-			TLSKey:  []byte(tlsKey),
-		},
 		&v1.QuayRegistry{
 			Spec: v1.QuayRegistrySpec{
 				Components: []v1.Component{
@@ -142,7 +122,6 @@ var processTests = []struct {
 	},
 	{
 		"volumeSizeDefault",
-		&quaycontext.QuayRegistryContext{},
 		&v1.QuayRegistry{
 			Spec: v1.QuayRegistrySpec{
 				Components: []v1.Component{
@@ -171,7 +150,6 @@ var processTests = []struct {
 	},
 	{
 		"volumeSizeOverridePostgres",
-		&quaycontext.QuayRegistryContext{},
 		&v1.QuayRegistry{
 			Spec: v1.QuayRegistrySpec{
 				Components: []v1.Component{
@@ -198,7 +176,6 @@ var processTests = []struct {
 	},
 	{
 		"volumeSizeOverrideClairPostgres",
-		&quaycontext.QuayRegistryContext{},
 		&v1.QuayRegistry{
 			Spec: v1.QuayRegistrySpec{
 				Components: []v1.Component{
@@ -226,7 +203,6 @@ var processTests = []struct {
 	},
 	{
 		"volumeSizeShrinkError",
-		&quaycontext.QuayRegistryContext{},
 		&v1.QuayRegistry{
 			Spec: v1.QuayRegistrySpec{
 				Components: []v1.Component{
@@ -254,7 +230,7 @@ func TestProcess(t *testing.T) {
 	for _, test := range processTests {
 
 		t.Run(test.name, func(t *testing.T) {
-			processedObj, err := Process(test.ctx, test.quay, test.obj)
+			processedObj, err := Process(test.quay, test.obj, false)
 			if test.expectedError != nil {
 				assert.Error(err, test.name)
 			} else {
