@@ -5,6 +5,7 @@ import (
 	"compress/gzip"
 	"context"
 	"crypto/tls"
+	"errors"
 	"fmt"
 	"io"
 	"io/ioutil"
@@ -17,12 +18,9 @@ import (
 	"testing"
 	"time"
 
-	"github.com/jackc/pgmock"
-
 	"github.com/jackc/pgconn"
+	"github.com/jackc/pgmock"
 	"github.com/jackc/pgproto3/v2"
-	errors "golang.org/x/xerrors"
-
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 )
@@ -481,6 +479,34 @@ func TestConnExecMultipleQueries(t *testing.T) {
 	ensureConnValid(t, pgConn)
 }
 
+func TestConnExecMultipleQueriesEagerFieldDescriptions(t *testing.T) {
+	t.Parallel()
+
+	pgConn, err := pgconn.Connect(context.Background(), os.Getenv("PGX_TEST_CONN_STRING"))
+	require.NoError(t, err)
+	defer closeConn(t, pgConn)
+
+	mrr := pgConn.Exec(context.Background(), "select 'Hello, world' as msg; select 1 as num")
+
+	require.True(t, mrr.NextResult())
+	require.Len(t, mrr.ResultReader().FieldDescriptions(), 1)
+	assert.Equal(t, []byte("msg"), mrr.ResultReader().FieldDescriptions()[0].Name)
+	_, err = mrr.ResultReader().Close()
+	require.NoError(t, err)
+
+	require.True(t, mrr.NextResult())
+	require.Len(t, mrr.ResultReader().FieldDescriptions(), 1)
+	assert.Equal(t, []byte("num"), mrr.ResultReader().FieldDescriptions()[0].Name)
+	_, err = mrr.ResultReader().Close()
+	require.NoError(t, err)
+
+	require.False(t, mrr.NextResult())
+
+	require.NoError(t, mrr.Close())
+
+	ensureConnValid(t, pgConn)
+}
+
 func TestConnExecMultipleQueriesError(t *testing.T) {
 	t.Parallel()
 
@@ -496,9 +522,18 @@ func TestConnExecMultipleQueriesError(t *testing.T) {
 		t.Errorf("unexpected error: %v", err)
 	}
 
-	assert.Len(t, results, 1)
-	assert.Len(t, results[0].Rows, 1)
-	assert.Equal(t, "1", string(results[0].Rows[0][0]))
+	if pgConn.ParameterStatus("crdb_version") != "" {
+		// CockroachDB starts the second query result set and then sends the divide by zero error.
+		require.Len(t, results, 2)
+		assert.Len(t, results[0].Rows, 1)
+		assert.Equal(t, "1", string(results[0].Rows[0][0]))
+		assert.Len(t, results[1].Rows, 0)
+	} else {
+		// PostgreSQL sends the divide by zero and never sends the second query result set.
+		require.Len(t, results, 1)
+		assert.Len(t, results[0].Rows, 1)
+		assert.Equal(t, "1", string(results[0].Rows[0][0]))
+	}
 
 	ensureConnValid(t, pgConn)
 }
@@ -510,6 +545,10 @@ func TestConnExecDeferredError(t *testing.T) {
 	require.NoError(t, err)
 	defer closeConn(t, pgConn)
 
+	if pgConn.ParameterStatus("crdb_version") != "" {
+		t.Skip("Server does not support deferred constraint (https://github.com/cockroachdb/cockroach/issues/31632)")
+	}
+
 	setupSQL := `create temporary table t (
 		id text primary key,
 		n int not null,
@@ -547,6 +586,11 @@ func TestConnExecContextCanceled(t *testing.T) {
 	err = multiResult.Close()
 	assert.True(t, pgconn.Timeout(err))
 	assert.True(t, pgConn.IsClosed())
+	select {
+	case <-pgConn.CleanupDone():
+	case <-time.After(5 * time.Second):
+		t.Fatal("Connection cleanup exceeded maximum time")
+	}
 }
 
 func TestConnExecContextPrecanceled(t *testing.T) {
@@ -573,7 +617,10 @@ func TestConnExecParams(t *testing.T) {
 	require.NoError(t, err)
 	defer closeConn(t, pgConn)
 
-	result := pgConn.ExecParams(context.Background(), "select $1::text", [][]byte{[]byte("Hello, world")}, nil, nil, nil)
+	result := pgConn.ExecParams(context.Background(), "select $1::text as msg", [][]byte{[]byte("Hello, world")}, nil, nil, nil)
+	require.Len(t, result.FieldDescriptions(), 1)
+	assert.Equal(t, []byte("msg"), result.FieldDescriptions()[0].Name)
+
 	rowCount := 0
 	for result.NextRow() {
 		rowCount += 1
@@ -594,6 +641,10 @@ func TestConnExecParamsDeferredError(t *testing.T) {
 	require.NoError(t, err)
 	defer closeConn(t, pgConn)
 
+	if pgConn.ParameterStatus("crdb_version") != "" {
+		t.Skip("Server does not support deferred constraint (https://github.com/cockroachdb/cockroach/issues/31632)")
+	}
+
 	setupSQL := `create temporary table t (
 		id text primary key,
 		n int not null,
@@ -680,6 +731,11 @@ func TestConnExecParamsCanceled(t *testing.T) {
 	assert.True(t, pgconn.Timeout(err))
 
 	assert.True(t, pgConn.IsClosed())
+	select {
+	case <-pgConn.CleanupDone():
+	case <-time.After(5 * time.Second):
+		t.Fatal("Connection cleanup exceeded maximum time")
+	}
 }
 
 func TestConnExecParamsPrecanceled(t *testing.T) {
@@ -717,6 +773,32 @@ func TestConnExecParamsEmptySQL(t *testing.T) {
 	ensureConnValid(t, pgConn)
 }
 
+// https://github.com/jackc/pgx/issues/859
+func TestResultReaderValuesHaveSameCapacityAsLength(t *testing.T) {
+	t.Parallel()
+
+	pgConn, err := pgconn.Connect(context.Background(), os.Getenv("PGX_TEST_CONN_STRING"))
+	require.NoError(t, err)
+	defer closeConn(t, pgConn)
+
+	result := pgConn.ExecParams(context.Background(), "select $1::text as msg", [][]byte{[]byte("Hello, world")}, nil, nil, nil)
+	require.Len(t, result.FieldDescriptions(), 1)
+	assert.Equal(t, []byte("msg"), result.FieldDescriptions()[0].Name)
+
+	rowCount := 0
+	for result.NextRow() {
+		rowCount += 1
+		assert.Equal(t, "Hello, world", string(result.Values()[0]))
+		assert.Equal(t, len(result.Values()[0]), cap(result.Values()[0]))
+	}
+	assert.Equal(t, 1, rowCount)
+	commandTag, err := result.Close()
+	assert.Equal(t, "SELECT 1", string(commandTag))
+	assert.NoError(t, err)
+
+	ensureConnValid(t, pgConn)
+}
+
 func TestConnExecPrepared(t *testing.T) {
 	t.Parallel()
 
@@ -724,13 +806,16 @@ func TestConnExecPrepared(t *testing.T) {
 	require.NoError(t, err)
 	defer closeConn(t, pgConn)
 
-	psd, err := pgConn.Prepare(context.Background(), "ps1", "select $1::text", nil)
+	psd, err := pgConn.Prepare(context.Background(), "ps1", "select $1::text as msg", nil)
 	require.NoError(t, err)
 	require.NotNil(t, psd)
 	assert.Len(t, psd.ParamOIDs, 1)
 	assert.Len(t, psd.Fields, 1)
 
 	result := pgConn.ExecPrepared(context.Background(), "ps1", [][]byte{[]byte("Hello, world")}, nil, nil)
+	require.Len(t, result.FieldDescriptions(), 1)
+	assert.Equal(t, []byte("msg"), result.FieldDescriptions()[0].Name)
+
 	rowCount := 0
 	for result.NextRow() {
 		rowCount += 1
@@ -790,14 +875,19 @@ func TestConnExecPreparedTooManyParams(t *testing.T) {
 	sql := "values" + strings.Join(params, ", ")
 
 	psd, err := pgConn.Prepare(context.Background(), "ps1", sql, nil)
-	require.NoError(t, err)
-	require.NotNil(t, psd)
-	assert.Len(t, psd.ParamOIDs, paramCount)
-	assert.Len(t, psd.Fields, 1)
+	if pgConn.ParameterStatus("crdb_version") != "" {
+		// CockroachDB rejects preparing a statement with more than 65535 parameters.
+		require.EqualError(t, err, "ERROR: more than 65535 arguments to prepared statement: 65536 (SQLSTATE 08P01)")
+	} else {
+		// PostgreSQL accepts preparing a statement with more than 65535 parameters and only fails when executing it through the extended protocol.
+		require.NoError(t, err)
+		require.NotNil(t, psd)
+		assert.Len(t, psd.ParamOIDs, paramCount)
+		assert.Len(t, psd.Fields, 1)
 
-	result := pgConn.ExecPrepared(context.Background(), "ps1", args, nil, nil).Read()
-	require.Error(t, result.Err)
-	require.Equal(t, "extended protocol limited to 65535 parameters", result.Err.Error())
+		result := pgConn.ExecPrepared(context.Background(), "ps1", args, nil, nil).Read()
+		require.EqualError(t, result.Err, "extended protocol limited to 65535 parameters")
+	}
 
 	ensureConnValid(t, pgConn)
 }
@@ -824,6 +914,11 @@ func TestConnExecPreparedCanceled(t *testing.T) {
 	assert.Equal(t, pgconn.CommandTag(nil), commandTag)
 	assert.True(t, pgconn.Timeout(err))
 	assert.True(t, pgConn.IsClosed())
+	select {
+	case <-pgConn.CleanupDone():
+	case <-time.After(5 * time.Second):
+		t.Fatal("Connection cleanup exceeded maximum time")
+	}
 }
 
 func TestConnExecPreparedPrecanceled(t *testing.T) {
@@ -906,6 +1001,10 @@ func TestConnExecBatchDeferredError(t *testing.T) {
 	require.NoError(t, err)
 	defer closeConn(t, pgConn)
 
+	if pgConn.ParameterStatus("crdb_version") != "" {
+		t.Skip("Server does not support deferred constraint (https://github.com/cockroachdb/cockroach/issues/31632)")
+	}
+
 	setupSQL := `create temporary table t (
 		id text primary key,
 		n int not null,
@@ -915,7 +1014,7 @@ func TestConnExecBatchDeferredError(t *testing.T) {
 	insert into t (id, n) values ('a', 1), ('b', 2), ('c', 3);`
 
 	_, err = pgConn.Exec(context.Background(), setupSQL).ReadAll()
-	assert.NoError(t, err)
+	require.NoError(t, err)
 
 	batch := &pgconn.Batch{}
 
@@ -997,6 +1096,10 @@ func TestConnExecBatchImplicitTransaction(t *testing.T) {
 	require.NoError(t, err)
 	defer closeConn(t, pgConn)
 
+	if pgConn.ParameterStatus("crdb_version") != "" {
+		t.Skip("Skipping due to known server issue: (https://github.com/cockroachdb/cockroach/issues/44803)")
+	}
+
 	_, err = pgConn.Exec(context.Background(), "create temporary table t(id int)").ReadAll()
 	require.NoError(t, err)
 
@@ -1086,6 +1189,10 @@ func TestConnOnNotice(t *testing.T) {
 	require.NoError(t, err)
 	defer closeConn(t, pgConn)
 
+	if pgConn.ParameterStatus("crdb_version") != "" {
+		t.Skip("Server does not support PL/PGSQL (https://github.com/cockroachdb/cockroach/issues/17511)")
+	}
+
 	multiResult := pgConn.Exec(context.Background(), `do $$
 begin
   raise notice 'hello, world';
@@ -1112,6 +1219,10 @@ func TestConnOnNotification(t *testing.T) {
 	require.NoError(t, err)
 	defer closeConn(t, pgConn)
 
+	if pgConn.ParameterStatus("crdb_version") != "" {
+		t.Skip("Server does not support LISTEN / NOTIFY (https://github.com/cockroachdb/cockroach/issues/41522)")
+	}
+
 	_, err = pgConn.Exec(context.Background(), "listen foo").ReadAll()
 	require.NoError(t, err)
 
@@ -1144,6 +1255,10 @@ func TestConnWaitForNotification(t *testing.T) {
 	require.NoError(t, err)
 	defer closeConn(t, pgConn)
 
+	if pgConn.ParameterStatus("crdb_version") != "" {
+		t.Skip("Server does not support LISTEN / NOTIFY (https://github.com/cockroachdb/cockroach/issues/41522)")
+	}
+
 	_, err = pgConn.Exec(context.Background(), "listen foo").ReadAll()
 	require.NoError(t, err)
 
@@ -1204,6 +1319,10 @@ func TestConnCopyToSmall(t *testing.T) {
 	require.NoError(t, err)
 	defer closeConn(t, pgConn)
 
+	if pgConn.ParameterStatus("crdb_version") != "" {
+		t.Skip("Server does support COPY TO")
+	}
+
 	_, err = pgConn.Exec(context.Background(), `create temporary table foo(
 		a int2,
 		b int4,
@@ -1242,6 +1361,10 @@ func TestConnCopyToLarge(t *testing.T) {
 	require.NoError(t, err)
 	defer closeConn(t, pgConn)
 
+	if pgConn.ParameterStatus("crdb_version") != "" {
+		t.Skip("Server does support COPY TO")
+	}
+
 	_, err = pgConn.Exec(context.Background(), `create temporary table foo(
 		a int2,
 		b int4,
@@ -1297,6 +1420,10 @@ func TestConnCopyToCanceled(t *testing.T) {
 	require.NoError(t, err)
 	defer closeConn(t, pgConn)
 
+	if pgConn.ParameterStatus("crdb_version") != "" {
+		t.Skip("Server does not support query cancellation (https://github.com/cockroachdb/cockroach/issues/41335)")
+	}
+
 	outputWriter := &bytes.Buffer{}
 
 	ctx, cancel := context.WithTimeout(context.Background(), 100*time.Millisecond)
@@ -1306,6 +1433,11 @@ func TestConnCopyToCanceled(t *testing.T) {
 	assert.Equal(t, pgconn.CommandTag(nil), res)
 
 	assert.True(t, pgConn.IsClosed())
+	select {
+	case <-pgConn.CleanupDone():
+	case <-time.After(5 * time.Second):
+		t.Fatal("Connection cleanup exceeded maximum time")
+	}
 }
 
 func TestConnCopyToPrecanceled(t *testing.T) {
@@ -1335,6 +1467,10 @@ func TestConnCopyFrom(t *testing.T) {
 	require.NoError(t, err)
 	defer closeConn(t, pgConn)
 
+	if pgConn.ParameterStatus("crdb_version") != "" {
+		t.Skip("Server does not fully support COPY FROM (https://www.cockroachlabs.com/docs/v20.2/copy-from.html)")
+	}
+
 	_, err = pgConn.Exec(context.Background(), `create temporary table foo(
 		a int4,
 		b varchar
@@ -1371,6 +1507,10 @@ func TestConnCopyFromCanceled(t *testing.T) {
 	require.NoError(t, err)
 	defer closeConn(t, pgConn)
 
+	if pgConn.ParameterStatus("crdb_version") != "" {
+		t.Skip("Server does not support query cancellation (https://github.com/cockroachdb/cockroach/issues/41335)")
+	}
+
 	_, err = pgConn.Exec(context.Background(), `create temporary table foo(
 		a int4,
 		b varchar
@@ -1397,6 +1537,11 @@ func TestConnCopyFromCanceled(t *testing.T) {
 	assert.Error(t, err)
 
 	assert.True(t, pgConn.IsClosed())
+	select {
+	case <-pgConn.CleanupDone():
+	case <-time.After(5 * time.Second):
+		t.Fatal("Connection cleanup exceeded maximum time")
+	}
 }
 
 func TestConnCopyFromPrecanceled(t *testing.T) {
@@ -1443,6 +1588,10 @@ func TestConnCopyFromGzipReader(t *testing.T) {
 	require.NoError(t, err)
 	defer closeConn(t, pgConn)
 
+	if pgConn.ParameterStatus("crdb_version") != "" {
+		t.Skip("Server does not fully support COPY FROM (https://www.cockroachlabs.com/docs/v20.2/copy-from.html)")
+	}
+
 	_, err = pgConn.Exec(context.Background(), `create temporary table foo(
 		a int4,
 		b varchar
@@ -1542,6 +1691,10 @@ func TestConnCopyFromNoticeResponseReceivedMidStream(t *testing.T) {
 	require.NoError(t, err)
 	defer closeConn(t, pgConn)
 
+	if pgConn.ParameterStatus("crdb_version") != "" {
+		t.Skip("Server does not support triggers (https://github.com/cockroachdb/cockroach/issues/28296)")
+	}
+
 	_, err = pgConn.Exec(ctx, `create temporary table sentences(
 		t text,
 		ts tsvector
@@ -1608,6 +1761,10 @@ func TestConnCancelRequest(t *testing.T) {
 	require.NoError(t, err)
 	defer closeConn(t, pgConn)
 
+	if pgConn.ParameterStatus("crdb_version") != "" {
+		t.Skip("Server does not support query cancellation (https://github.com/cockroachdb/cockroach/issues/41335)")
+	}
+
 	multiResult := pgConn.Exec(context.Background(), "select 'Hello, world', pg_sleep(2)")
 
 	// This test flickers without the Sleep. It appears that since Exec only sends the query and returns without awaiting a
@@ -1647,6 +1804,11 @@ func TestConnContextCanceledCancelsRunningQueryOnServer(t *testing.T) {
 	err = multiResult.Close()
 	assert.True(t, pgconn.Timeout(err))
 	assert.True(t, pgConn.IsClosed())
+	select {
+	case <-pgConn.CleanupDone():
+	case <-time.After(5 * time.Second):
+		t.Fatal("Connection cleanup exceeded maximum time")
+	}
 
 	otherConn, err := pgconn.Connect(context.Background(), os.Getenv("PGX_TEST_CONN_STRING"))
 	require.NoError(t, err)
@@ -1750,6 +1912,76 @@ func TestConnCloseWhileCancellableQueryInProgress(t *testing.T) {
 
 	closeCtx, _ := context.WithCancel(context.Background())
 	pgConn.Close(closeCtx)
+	select {
+	case <-pgConn.CleanupDone():
+	case <-time.After(5 * time.Second):
+		t.Fatal("Connection cleanup exceeded maximum time")
+	}
+}
+
+// https://github.com/jackc/pgx/issues/800
+func TestFatalErrorReceivedAfterCommandComplete(t *testing.T) {
+	t.Parallel()
+
+	steps := pgmock.AcceptUnauthenticatedConnRequestSteps()
+	steps = append(steps, pgmock.ExpectAnyMessage(&pgproto3.Parse{}))
+	steps = append(steps, pgmock.ExpectAnyMessage(&pgproto3.Bind{}))
+	steps = append(steps, pgmock.ExpectAnyMessage(&pgproto3.Describe{}))
+	steps = append(steps, pgmock.ExpectAnyMessage(&pgproto3.Execute{}))
+	steps = append(steps, pgmock.ExpectAnyMessage(&pgproto3.Sync{}))
+	steps = append(steps, pgmock.SendMessage(&pgproto3.RowDescription{Fields: []pgproto3.FieldDescription{
+		{Name: []byte("mock")},
+	}}))
+	steps = append(steps, pgmock.SendMessage(&pgproto3.CommandComplete{CommandTag: []byte("SELECT 0")}))
+	steps = append(steps, pgmock.SendMessage(&pgproto3.ErrorResponse{Severity: "FATAL", Code: "57P01"}))
+
+	script := &pgmock.Script{Steps: steps}
+
+	ln, err := net.Listen("tcp", "127.0.0.1:")
+	require.NoError(t, err)
+	defer ln.Close()
+
+	serverErrChan := make(chan error, 1)
+	go func() {
+		defer close(serverErrChan)
+
+		conn, err := ln.Accept()
+		if err != nil {
+			serverErrChan <- err
+			return
+		}
+		defer conn.Close()
+
+		err = conn.SetDeadline(time.Now().Add(5 * time.Second))
+		if err != nil {
+			serverErrChan <- err
+			return
+		}
+
+		err = script.Run(pgproto3.NewBackend(pgproto3.NewChunkReader(conn), conn))
+		if err != nil {
+			serverErrChan <- err
+			return
+		}
+	}()
+
+	parts := strings.Split(ln.Addr().String(), ":")
+	host := parts[0]
+	port := parts[1]
+	connStr := fmt.Sprintf("sslmode=disable host=%s port=%s", host, port)
+
+	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
+	defer cancel()
+	conn, err := pgconn.Connect(ctx, connStr)
+	require.NoError(t, err)
+
+	rr := conn.ExecParams(ctx, "mocked...", nil, nil, nil, nil)
+
+	for rr.NextRow() {
+	}
+
+	_, err = rr.Close()
+	require.Error(t, err)
 }
 
 func Example() {
