@@ -9,9 +9,11 @@ import (
 	"math"
 	"os"
 	"reflect"
+	"regexp"
 	"testing"
 	"time"
 
+	"github.com/Masterminds/semver/v3"
 	"github.com/jackc/pgconn"
 	"github.com/jackc/pgx/v4"
 	"github.com/jackc/pgx/v4/stdlib"
@@ -30,6 +32,53 @@ func closeDB(t testing.TB, db *sql.DB) {
 	require.NoError(t, err)
 }
 
+func skipCockroachDB(t testing.TB, db *sql.DB, msg string) {
+	conn, err := db.Conn(context.Background())
+	require.NoError(t, err)
+	defer conn.Close()
+
+	err = conn.Raw(func(driverConn interface{}) error {
+		conn := driverConn.(*stdlib.Conn).Conn()
+		if conn.PgConn().ParameterStatus("crdb_version") != "" {
+			t.Skip(msg)
+		}
+		return nil
+	})
+	require.NoError(t, err)
+}
+
+func skipPostgreSQLVersion(t testing.TB, db *sql.DB, constraintStr, msg string) {
+	conn, err := db.Conn(context.Background())
+	require.NoError(t, err)
+	defer conn.Close()
+
+	err = conn.Raw(func(driverConn interface{}) error {
+		conn := driverConn.(*stdlib.Conn).Conn()
+		serverVersionStr := conn.PgConn().ParameterStatus("server_version")
+		serverVersionStr = regexp.MustCompile(`^[0-9.]+`).FindString(serverVersionStr)
+		// if not PostgreSQL do nothing
+		if serverVersionStr == "" {
+			return nil
+		}
+
+		serverVersion, err := semver.NewVersion(serverVersionStr)
+		if err != nil {
+			return err
+		}
+
+		c, err := semver.NewConstraint(constraintStr)
+		if err != nil {
+			return err
+		}
+
+		if c.Check(serverVersion) {
+			t.Skip(msg)
+		}
+		return nil
+	})
+	require.NoError(t, err)
+}
+
 func testWithAndWithoutPreferSimpleProtocol(t *testing.T, f func(t *testing.T, db *sql.DB)) {
 	t.Run("SimpleProto",
 		func(t *testing.T) {
@@ -118,6 +167,8 @@ func TestNormalLifeCycle(t *testing.T) {
 	db := openDB(t)
 	defer closeDB(t, db)
 
+	skipCockroachDB(t, db, "Server issues incorrect ParameterDescription (https://github.com/cockroachdb/cockroach/issues/60907)")
+
 	stmt := prepareStmt(t, db, "select 'foo', n from generate_series($1::int, $2::int) n")
 	defer closeStmt(t, stmt)
 
@@ -179,6 +230,8 @@ func TestQueryCloseRowsEarly(t *testing.T) {
 	db := openDB(t)
 	defer closeDB(t, db)
 
+	skipCockroachDB(t, db, "Server issues incorrect ParameterDescription (https://github.com/cockroachdb/cockroach/issues/60907)")
+
 	stmt := prepareStmt(t, db, "select 'foo', n from generate_series($1::int, $2::int) n")
 	defer closeStmt(t, stmt)
 
@@ -234,6 +287,8 @@ func TestConnExec(t *testing.T) {
 
 func TestConnQuery(t *testing.T) {
 	testWithAndWithoutPreferSimpleProtocol(t, func(t *testing.T, db *sql.DB) {
+		skipCockroachDB(t, db, "Server issues incorrect ParameterDescription (https://github.com/cockroachdb/cockroach/issues/60907)")
+
 		rows, err := db.Query("select 'foo', n from generate_series($1::int, $2::int) n", int32(1), int32(10))
 		require.NoError(t, err)
 
@@ -321,6 +376,17 @@ func TestConnQueryFailure(t *testing.T) {
 	})
 }
 
+func TestConnSimpleSlicePassThrough(t *testing.T) {
+	testWithAndWithoutPreferSimpleProtocol(t, func(t *testing.T, db *sql.DB) {
+		skipCockroachDB(t, db, "Server does not support cardinality function")
+
+		var n int64
+		err := db.QueryRow("select cardinality($1::text[])", []string{"a", "b", "c"}).Scan(&n)
+		require.NoError(t, err)
+		assert.EqualValues(t, 3, n)
+	})
+}
+
 // Test type that pgx would handle natively in binary, but since it is not a
 // database/sql native type should be passed through as a string
 func TestConnQueryRowPgxBinary(t *testing.T) {
@@ -337,6 +403,8 @@ func TestConnQueryRowPgxBinary(t *testing.T) {
 
 func TestConnQueryRowUnknownType(t *testing.T) {
 	testWithAndWithoutPreferSimpleProtocol(t, func(t *testing.T, db *sql.DB) {
+		skipCockroachDB(t, db, "Server does not support point type")
+
 		sql := "select $1::point"
 		expected := "(1,2)"
 		var actual string
@@ -354,12 +422,12 @@ func TestConnQueryJSONIntoByteSlice(t *testing.T) {
 			body json not null
 		);
 
-		insert into docs(body) values('{"foo":"bar"}');
+		insert into docs(body) values('{"foo": "bar"}');
 `)
 		require.NoError(t, err)
 
 		sql := `select * from docs`
-		expected := []byte(`{"foo":"bar"}`)
+		expected := []byte(`{"foo": "bar"}`)
 		var actual []byte
 
 		err = db.QueryRow(sql).Scan(&actual)
@@ -390,7 +458,7 @@ func TestConnExecInsertByteSliceIntoJSON(t *testing.T) {
 `)
 	require.NoError(t, err)
 
-	expected := []byte(`{"foo":"bar"}`)
+	expected := []byte(`{"foo": "bar"}`)
 
 	_, err = db.Exec(`insert into docs(body) values($1)`, expected)
 	require.NoError(t, err)
@@ -443,6 +511,8 @@ func TestTransactionLifeCycle(t *testing.T) {
 
 func TestConnBeginTxIsolation(t *testing.T) {
 	testWithAndWithoutPreferSimpleProtocol(t, func(t *testing.T, db *sql.DB) {
+		skipCockroachDB(t, db, "Server always uses serializable isolation level")
+
 		var defaultIsoLevel string
 		err := db.QueryRow("show transaction_isolation").Scan(&defaultIsoLevel)
 		require.NoError(t, err)
@@ -595,6 +665,8 @@ func TestConnRaw(t *testing.T) {
 // https://github.com/jackc/pgx/issues/673
 func TestReleaseConnWithTxInProgress(t *testing.T) {
 	testWithAndWithoutPreferSimpleProtocol(t, func(t *testing.T, db *sql.DB) {
+		skipCockroachDB(t, db, "Server does not support backend PID")
+
 		c1, err := stdlib.AcquireConn(db)
 		require.NoError(t, err)
 
@@ -694,15 +766,15 @@ func TestConnQueryContextFailureRetry(t *testing.T) {
 
 func TestRowsColumnTypeDatabaseTypeName(t *testing.T) {
 	testWithAndWithoutPreferSimpleProtocol(t, func(t *testing.T, db *sql.DB) {
-		rows, err := db.Query("select * from generate_series(1,10) n")
+		rows, err := db.Query("select 42::bigint")
 		require.NoError(t, err)
 
 		columnTypes, err := rows.ColumnTypes()
 		require.NoError(t, err)
 		require.Len(t, columnTypes, 1)
 
-		if columnTypes[0].DatabaseTypeName() != "INT4" {
-			t.Errorf("columnTypes[0].DatabaseTypeName() => %v, want %v", columnTypes[0].DatabaseTypeName(), "INT4")
+		if columnTypes[0].DatabaseTypeName() != "INT8" {
+			t.Errorf("columnTypes[0].DatabaseTypeName() => %v, want %v", columnTypes[0].DatabaseTypeName(), "INT8")
 		}
 
 		err = rows.Close()
@@ -753,6 +825,8 @@ func TestStmtQueryContextSuccess(t *testing.T) {
 	db := openDB(t)
 	defer closeDB(t, db)
 
+	skipCockroachDB(t, db, "Server issues incorrect ParameterDescription (https://github.com/cockroachdb/cockroach/issues/60907)")
+
 	stmt, err := db.Prepare("select * from generate_series(1,$1::int4) n")
 	require.NoError(t, err)
 	defer stmt.Close()
@@ -792,7 +866,7 @@ func TestRowsColumnTypes(t *testing.T) {
 		}{
 			{
 				Name:     "a",
-				TypeName: "INT4",
+				TypeName: "INT8",
 				Length: struct {
 					Len int64
 					OK  bool
@@ -809,7 +883,7 @@ func TestRowsColumnTypes(t *testing.T) {
 					Scale:     0,
 					OK:        false,
 				},
-				ScanType: reflect.TypeOf(int32(0)),
+				ScanType: reflect.TypeOf(int64(0)),
 			}, {
 				Name:     "bar",
 				TypeName: "TEXT",
@@ -873,7 +947,7 @@ func TestRowsColumnTypes(t *testing.T) {
 			},
 		}
 
-		rows, err := db.Query("SELECT 1 AS a, text 'bar' AS bar, 1.28::numeric(9, 2) AS dec, '12:00:00'::timetz as d")
+		rows, err := db.Query("SELECT 1::bigint AS a, text 'bar' AS bar, 1.28::numeric(9, 2) AS dec, '12:00:00'::timetz as d")
 		require.NoError(t, err)
 
 		columns, err := rows.ColumnTypes()
@@ -914,6 +988,8 @@ func TestRowsColumnTypes(t *testing.T) {
 
 func TestQueryLifeCycle(t *testing.T) {
 	testWithAndWithoutPreferSimpleProtocol(t, func(t *testing.T, db *sql.DB) {
+		skipCockroachDB(t, db, "Server issues incorrect ParameterDescription (https://github.com/cockroachdb/cockroach/issues/60907)")
+
 		rows, err := db.Query("SELECT 'foo', n FROM generate_series($1::int, $2::int) n WHERE 3 = $3", 1, 10, 3)
 		require.NoError(t, err)
 
@@ -1005,3 +1081,43 @@ func TestRegisterConnConfig(t *testing.T) {
 	assert.Equal(t, "Query", l.msg)
 	assert.Equal(t, "select 1", l.data["sql"])
 }
+
+// https://github.com/jackc/pgx/issues/958
+func TestConnQueryRowConstraintErrors(t *testing.T) {
+	testWithAndWithoutPreferSimpleProtocol(t, func(t *testing.T, db *sql.DB) {
+		skipPostgreSQLVersion(t, db, "< 11", "Test requires PG 11+")
+		skipCockroachDB(t, db, "Server does not support deferred constraint (https://github.com/cockroachdb/cockroach/issues/31632)")
+
+		_, err := db.Exec(`create temporary table defer_test (
+			id text primary key,
+			n int not null, unique (n),
+			unique (n) deferrable initially deferred )`)
+		require.NoError(t, err)
+
+		_, err = db.Exec(`drop function if exists test_trigger cascade`)
+		require.NoError(t, err)
+
+		_, err = db.Exec(`create function test_trigger() returns trigger language plpgsql as $$
+		begin
+		if new.n = 4 then
+			raise exception 'n cant be 4!';
+		end if;
+		return new;
+	end$$`)
+		require.NoError(t, err)
+
+		_, err = db.Exec(`create constraint trigger test
+			after insert or update on defer_test
+			deferrable initially deferred
+			for each row
+			execute function test_trigger()`)
+		require.NoError(t, err)
+
+		_, err = db.Exec(`insert into defer_test (id, n) values ('a', 1), ('b', 2), ('c', 3)`)
+		require.NoError(t, err)
+
+		var id string
+		err = db.QueryRow(`insert into defer_test (id, n) values ('e', 4) returning id`).Scan(&id)
+		assert.Error(t, err)
+	})
+}
